@@ -143,7 +143,7 @@ QString a1time::toPretty() const
 	if (!isValid())
 		 return QObject::tr("Broken / Invalid");
 
-	return toLocalTime().toString(Qt::SystemLocaleLongDate);
+	return toLocalTime().toString(Qt::DefaultLocaleLongDate);
 }
 
 QString a1time::toPrettyGMT() const
@@ -165,6 +165,15 @@ QString a1time::toPlain() const
 	return toUTC().toString(GEN_FORMAT);
 }
 
+QString a1time::toPlainUTC() const
+{
+	if (isUndefined())
+		return QString(UNDEFINED_DATE);
+	if (!isValid())
+		 return QString("Broken-InvalidZ");
+	return toUTC().toString(UTC_FORMAT);
+}
+
 QString a1time::toSortable() const
 {
 	if (isUndefined())
@@ -183,6 +192,7 @@ QDateTime a1time::now(int delta)
 void a1time::d2i(QByteArray &ba)
 {
 	ASN1_TIME *n = (ASN1_TIME*)d2i_bytearray( D2I_VOID(d2i_ASN1_TIME), ba);
+	openssl_error();
 	if (n) {
 		from_asn1(n);
 		ASN1_TIME_free(n);
