@@ -16,13 +16,14 @@
 #endif
 
 #include <qglobal.h>
-#include "local.h"
-
-#ifdef WIN32
-#include <windows.h>
-#endif
-
 #include <openssl/opensslv.h>
+#ifndef QMAKE
+#include "local.h"
+#else
+#define PREFIX "/usr/local"
+#define ETC "/etc"
+#define DOCDIR "/usr/local/doc/xca"
+#endif
 
 #define CCHAR(x) qPrintable(x)
 #endif
@@ -33,10 +34,10 @@
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
 
  #define nativeSeparator(s) QDir::toNativeSeparators(s)
- #define XCA_INFO(msg) QMessageBox::information(NULL, XCA_TITLE, msg.toHtmlEscaped())
- #define XCA_WARN(msg) QMessageBox::warning(NULL, XCA_TITLE, msg.toHtmlEscaped())
- #define XCA_YESNO(msg) (QMessageBox::question(NULL, XCA_TITLE, msg.toHtmlEscaped(), QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes)
- #define XCA_OKCANCEL(msg) (QMessageBox::warning(NULL, XCA_TITLE, msg.toHtmlEscaped(), QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Ok)
+ #define XCA_INFO(msg) QMessageBox::information(NULL, XCA_TITLE, QString(msg).toHtmlEscaped())
+ #define XCA_WARN(msg) QMessageBox::warning(NULL, XCA_TITLE, QString(msg).toHtmlEscaped())
+ #define XCA_YESNO(msg) (QMessageBox::question(NULL, XCA_TITLE, QString(msg).toHtmlEscaped(), QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes)
+ #define XCA_OKCANCEL(msg) (QMessageBox::warning(NULL, XCA_TITLE, QString(msg).toHtmlEscaped(), QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Ok)
 
 #else
 
@@ -51,3 +52,17 @@
 
 #define ARRAY_SIZE(x) (sizeof(x)/sizeof((x)[0]))
 #define MIN(a,b) ((a)<(b)) ? (a) : (b)
+
+#if Q_BYTE_ORDER == Q_BIG_ENDIAN
+#define xhtonl(x) (x)
+#define xntohl(x) (x)
+#define xhtons(x) (x)
+#define xntohs(x) (x)
+#elif Q_BYTE_ORDER == Q_LITTLE_ENDIAN
+#define xhtonl(x) (__builtin_bswap32(x))
+#define xntohl(x) (__builtin_bswap32(x))
+#define xhtons(x) (__builtin_bswap16(x))
+#define xntohs(x) (__builtin_bswap16(x))
+#else
+	# error "What kind of system is this?"
+#endif
