@@ -5,12 +5,6 @@
  * All rights reserved.
  */
 
-#ifdef WIN32
-#include <windows.h>
-#else
-#include <netinet/in.h>
-#endif
-
 #include "db_base.h"
 #include "func.h"
 #include "exception.h"
@@ -19,6 +13,7 @@
 #include <QClipboard>
 #include <QDir>
 #include <QDebug>
+#include <QMimeData>
 #include "widgets/MainWindow.h"
 #include "widgets/ImportMulti.h"
 
@@ -103,12 +98,12 @@ int db_base::handleBadEntry(unsigned char *p, db_header_t *head)
                         arg(s).arg(strerror(errno)), class_name);
 	}
 
-	h.magic   = ntohl(head->magic);
-	h.len     = ntohl(head->len);
-	h.headver = ntohs(head->headver);
-	h.type    = ntohs(head->type);
-	h.version = ntohs(head->version);
-	h.flags   = ntohs(head->flags);
+	h.magic   = xntohl(head->magic);
+	h.len     = xntohl(head->len);
+	h.headver = xntohs(head->headver);
+	h.type    = xntohs(head->type);
+	h.version = xntohs(head->version);
+	h.flags   = xntohs(head->flags);
 	memcpy(h.name, head->name, NAMELEN);
 
 	l = fwrite(&h, sizeof h, 1, fp);
@@ -264,6 +259,8 @@ void db_base::initHeaderView(QHeaderView *hv)
 void db_base::sortIndicatorChanged(int logicalIndex, Qt::SortOrder order)
 {
 	int max = allHeaders.count();
+	if (!isValidCol(logicalIndex))
+		return;
 	for (int i=0; i<max; i++) {
 		allHeaders[i]->sortIndicator = -1;
 	}
@@ -286,7 +283,7 @@ void db_base::insertPKI(pki_base *pki)
 	emit columnsContentChanged();
 }
 
-QString db_base::pem2QString(QModelIndexList indexes)
+QString db_base::pem2QString(QModelIndexList indexes) const
 {
 	exportType::etype format;
 	QString msg;
@@ -295,6 +292,8 @@ QString db_base::pem2QString(QModelIndexList indexes)
 	foreach(QModelIndex idx, indexes) {
 		long l;
 		const char *p;
+		if (idx.column() != 0)
+			continue;
 		BIO *bio = BIO_new(BIO_s_mem());
 		pki_base *pki = static_cast<pki_base*>
 				(idx.internalPointer());
@@ -307,15 +306,14 @@ QString db_base::pem2QString(QModelIndexList indexes)
 	return msg;
 }
 
-void db_base::pem2clipboard(QModelIndexList indexes)
+void db_base::pem2clipboard(QModelIndexList indexes) const
 {
 	QString msg = pem2QString(indexes);
 	QClipboard *cb = QApplication::clipboard();
 
 	if (cb->supportsSelection())
 		cb->setText(msg, QClipboard::Selection);
-	else
-		cb->setText(msg);
+	cb->setText(msg);
 }
 
 void db_base::deletePKI(QModelIndex idx)
@@ -535,6 +533,8 @@ static QVariant getHeaderViewInfo(dbheader *h)
 QVariant db_base::headerData(int section, Qt::Orientation orientation,
 		int role) const
 {
+	if (!isValidCol(section))
+		return QVariant();
 	if (orientation == Qt::Horizontal) {
 		switch (role) {
 		case Qt::DisplayRole:
@@ -551,10 +551,12 @@ Qt::ItemFlags db_base::flags(const QModelIndex &index) const
 	if (!index.isValid())
 		return Qt::ItemIsEnabled;
 
+	Qt::ItemFlags defaultFlags = QAbstractItemModel::flags(index);
+	defaultFlags |= Qt::ItemIsDragEnabled;
+
 	if (index.column() == 0)
-		return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;
-	else
-		return QAbstractItemModel::flags(index);
+		return defaultFlags |= Qt::ItemIsEditable;
+	return defaultFlags;
 }
 
 bool db_base::setData(const QModelIndex &index, const QVariant &value, int role)
@@ -629,7 +631,7 @@ void db_base::store(QModelIndexList indexes)
 	QString fn = mainwin->getPath() + QDir::separator() + "export.pem";
 	QString s = QFileDialog::getSaveFileName(mainwin,
 		tr("Save %1 items in one file as").arg(indexes.size()), fn,
-		tr("PEM Files( *.pem );; All files ( * )"));
+		tr("PEM files ( *.pem );; All files ( * )"));
 	if (s.isEmpty())
 		return;
 	s = nativeSeparator(s);
@@ -641,7 +643,9 @@ void db_base::store(QModelIndexList indexes)
 			throw errorEx(tr("Error opening file: '%1': %2").
 				arg(s).arg(strerror(errno)), class_name);
 		}
-		fwrite(pem.toLatin1(), pem.size(), 1, fp);
+		if (fwrite(pem.toLatin1(), pem.size(), 1, fp)) {
+			/* IGNORE_RESULT */
+		}
 		fclose(fp);
 	}
 	catch (errorEx &err) {
@@ -651,6 +655,8 @@ void db_base::store(QModelIndexList indexes)
 
 bool db_base::columnHidden(int col) const
 {
+	if (!isValidCol(col))
+		return true;
 	if (pki_x509::disable_netscape &&
 	    allHeaders[col]->type == dbheader::hd_v3ext_ns)
 		return true;
@@ -668,5 +674,23 @@ void db_base::columnResetDefaults()
 
 bool db_base::isNumericCol(int col) const
 {
-	return allHeaders[col]->isNumeric();
+	return isValidCol(col) ? allHeaders[col]->isNumeric() : false;
+}
+
+bool db_base::isValidCol(int col) const
+{
+	return col >= allHeaders.size() || col < 0 ? false : true;
+}
+
+QMimeData *db_base::mimeData(const QModelIndexList &indexes) const
+{
+	QString data = pem2QString(indexes);
+
+	if (data.isEmpty())
+		return NULL;
+
+	QMimeData *mimeData = new QMimeData();
+	mimeData->setText(data.toLatin1());
+	mimeData->setData(X_XCA_DRAG_DATA, QByteArray());
+	return mimeData;
 }
