@@ -11,25 +11,31 @@
 #include "widgets/MainWindow.h"
 #include "widgets/CrlDetail.h"
 #include "widgets/NewCrl.h"
-#include <QtGui/QMessageBox>
-#include <QtGui/QContextMenuEvent>
+#include <QMessageBox>
+#include <QContextMenuEvent>
+#include <QInputDialog>
 #include "ui_NewCrl.h"
 
 db_crl::db_crl(QString db, MainWindow *mw)
 	:db_x509name(db,mw)
 {
-	allHeaders <<
-		new dbheader(HD_crl_signer,	true, tr("Signer"),
+	class_name = "crls";
+	pkitype << revocation;
+	updateHeaders();
+	loadContainer();
+}
+
+dbheaderList db_crl::getHeaders()
+{
+	dbheaderList h = db_x509name::getHeaders();
+	h <<	new dbheader(HD_crl_signer,	true, tr("Signer"),
 			tr("Internal name of the signer")) <<
 		new dbheader(HD_crl_revoked,	true, tr("No. revoked"),
 			tr("Number of revoked certificates")) <<
 		new dbheader(HD_crl_lastUpdate, false,tr("Last update")) <<
 		new dbheader(HD_crl_nextUpdate,	true, tr("Next update")) <<
 		new dbheader(HD_crl_crlnumber,	false,tr("CRL number"));
-
-	class_name = "crls";
-	pkitype << revokation;
-	loadContainer();
+	return h;
 }
 
 pki_base *db_crl::newPKI(db_header_t *)
@@ -45,17 +51,19 @@ void db_crl::load()
 
 void db_crl::revokeCerts(pki_crl *crl)
 {
-	int numc, i;
+	x509revList revlist;
+
 	if (!mainwin->certs)
 		return;
-	x509rev revok;
 	pki_x509 *signer = crl->getIssuer();
 	if (!signer)
 		return;
-	numc = crl->numRev();
-	for (i=0; i<numc; i++) {
-		revok = crl->getRev(i);
-		mainwin->certs->revokeCert(revok, signer);
+	revlist = crl->getRevList();
+	signer->mergeRevList(revlist);
+	foreach(x509rev revok, revlist) {
+		pki_x509 *crt = signer->getBySerial(revok.getSerial());
+		if (crt)
+			crt->setRevoked(revok);
 	}
 }
 
@@ -71,22 +79,31 @@ void db_crl::inToCont(pki_base *pki)
 {
 	pki_crl *crl = (pki_crl *)pki;
 	if (crl->getIssuer() == NULL) {
-		pki_x509 *iss = NULL, *last = NULL;
+		pki_x509 *iss = NULL, *last = NULL, *newest = NULL;
 		x509name issname = crl->getSubject();
-		while ((iss = mainwin->certs->getBySubject(issname, last)) != NULL) {
-			pki_key *key = iss->getPubKey();
-			if (key) {
-				if (crl->verify(key)) {
-					delete key;
-					break;
-				}
-				delete key;
-			}
+		while (1) {
+			iss = mainwin->certs->getBySubject(issname, last);
+			if (!iss)
+				break;
 			last = iss;
+			pki_key *key = iss->getPubKey();
+			if (!key)
+				continue;
+
+			if (!crl->verify(key)) {
+				delete key;
+				continue;
+			}
+			delete key;
+			if (!newest) {
+				newest = iss;
+			} else {
+				if (newest->getNotAfter() < iss->getNotAfter())
+					newest = iss;
+			}
 		}
-		crl->setIssuer(iss);
+		crl->setIssuer(newest);
 	}
-	revokeCerts(crl);
 	db_base::inToCont(pki);
 }
 
@@ -95,11 +112,15 @@ pki_base *db_crl::insert(pki_base *item)
 	pki_crl *crl = (pki_crl *)item;
 	pki_crl *oldcrl = (pki_crl *)getByReference(crl);
 	if (oldcrl) {
-		XCA_INFO(tr("The revokation list already exists in the database as:\n'%1'\nand so it was not imported").arg(oldcrl->getIntName()));
+		XCA_INFO(tr("The revocation list already exists in the database as:\n'%1'\nand so it was not imported").arg(oldcrl->getIntName()));
 		delete(crl);
 		return NULL;
 	}
 	insertPKI(crl);
+	revokeCerts(crl);
+	pki_x509 *issuer = crl->getIssuer();
+	if (issuer)
+		mainwin->certs->updateAfterCrlLoad(issuer);
 	return crl;
 }
 
@@ -118,53 +139,108 @@ void db_crl::showPki(pki_base *pki)
 	}
 }
 
-void db_crl::store()
+void db_crl::store(QModelIndex index)
 {
-	if (!currentIdx.isValid())
-		return;
+	QList<exportType> types;
 
-	pki_crl *crl = static_cast<pki_crl*>(currentIdx.internalPointer());
+	if (!index.isValid())
+		return;
+	pki_crl *crl = static_cast<pki_crl*>(index.internalPointer());
 	if (!crl)
 		return;
 
-	QString fn = mainwin->getPath() + QDir::separator() +
-		crl->getUnderlinedName() + ".pem";
-	ExportDer *dlg = new ExportDer(mainwin, fn,
-			tr("CRL ( *.pem *.der *.crl )"));
-	dlg->image->setPixmap(*MainWindow::revImg);
-	dlg->label->setText(tr("Revocation list export"));
-	int dlgret = dlg->exec();
-
-	if (!dlgret) {
+	types << exportType(exportType::PEM, "pem", "PEM") <<
+			exportType(exportType::DER, "der", "DER");
+	ExportDialog *dlg = new ExportDialog(mainwin,
+			tr("Revocation list export"),
+			tr("CRL ( *.pem *.der *.crl )"), crl,
+			MainWindow::revImg, types);
+	if (!dlg->exec()) {
 		delete dlg;
 		return;
 	}
 	QString fname = dlg->filename->text();
-	bool pem = dlg->exportFormat->currentIndex() == 0 ? true : false;
-	delete dlg;
-	if (fname == "") {
-		return;
-	}
-	mainwin->setPath(fname.mid(0, fname.lastIndexOf(QRegExp("[/\\\\]")) ));
+
 	try {
-		crl->writeCrl(fname, pem);
+		crl->writeCrl(fname, dlg->type() == exportType::PEM);
 	}
 	catch (errorEx &err) {
 		mainwin->Error(err);
 	}
+	delete dlg;
 }
 
-pki_crl *db_crl::newItem(pki_x509 *cert)
+void db_crl::updateRevocations(pki_x509 *cert)
+{
+	x509name issname = cert->getSubject();
+	x509revList revlist;
+	pki_crl *latest = NULL;
+
+	FOR_ALL_pki(crl, pki_crl) {
+		if (!(issname == crl->getSubject()))
+			continue;
+		pki_key *key = cert->getPubKey();
+		if (!key)
+			continue;
+		if (!crl->verify(key)) {
+			delete key;
+			continue;
+		}
+		delete key;
+		pki_x509 *old = crl->getIssuer();
+		if (!old) {
+			crl->setIssuer(cert);
+		} else if (old != cert) {
+			if (old->getNotAfter() < cert->getNotAfter())
+				crl->setIssuer(cert);
+		}
+		if (!latest || (latest->getCrlNumber() < crl->getCrlNumber()))
+			latest = crl;
+	}
+	if (latest) {
+		revlist = latest->getRevList();
+		cert->mergeRevList(revlist);
+		cert->setCrlNumber(latest->getCrlNumber());
+	}
+}
+
+void db_crl::newItem()
+{
+	bool ok = false;
+	QStringList sl = mainwin->certs->getSignerDesc();
+	QString ca;
+
+	switch (sl.size()) {
+	case 0:
+		XCA_INFO(tr("There are no CA certificates for CRL generation"));
+		break;
+	case 1:
+		ca = sl[0];
+		ok = true;
+		break;
+	default:
+		ca = QInputDialog::getItem(mainwin, XCA_TITLE,
+			tr("Select CA certificate"), sl, 0, false, &ok, 0);
+	}
+	if (!ok)
+		return;
+
+	pki_x509 *cert = static_cast<pki_x509*>
+		(mainwin->certs->getByName(ca));
+	newItem(cert);
+}
+
+void db_crl::newItem(pki_x509 *cert)
 {
 	if (!cert)
-		return NULL;
+		return;
 
 	pki_crl *crl = NULL;
 	NewCrl *dlg = new NewCrl(mainwin, cert);
 
 	if (!dlg->exec()) {
 		delete dlg;
-		return NULL;
+		return;
 	}
 	try {
 		x509v3ext e;
@@ -175,13 +251,9 @@ pki_crl *db_crl::newItem(pki_x509 *cert)
 		crl = new pki_crl();
 		crl->createCrl(cert->getIntName(), cert);
 
-		QList<pki_x509*> list = mainwin->certs->getIssuedCerts(cert);
-		bool reason = dlg->revokationReasons->isChecked();
-		for (int i =0; i<list.size(); i++) {
-			if (list.at(i)->isRevoked() ) {
-				crl->addRev(list.at(i)->getRev(reason));
-			}
-		}
+		bool withReason = dlg->revocationReasons->isChecked();
+		foreach(x509rev rev, cert->revList)
+			crl->addRev(rev, withReason);
 
 		if (dlg->authKeyId->isChecked()) {
 			crl->addV3ext(e.create(NID_authority_key_identifier,
@@ -193,8 +265,11 @@ pki_crl *db_crl::newItem(pki_x509 *cert)
 					"issuer:copy", &ext_ctx));
 			}
 		}
-		if (dlg->crlNumber->isChecked()) {
-			crl->setCrlNumber(cert->getIncCrlNumber());
+		if (dlg->setCrlNumber->isChecked()) {
+			a1int num;
+			num.setDec(dlg->crlNumber->text());
+			crl->setCrlNumber(num);
+			cert->setCrlNumber(num);
 		}
 		crl->setLastUpdate(dlg->lastUpdate->getDate());
 		crl->setNextUpdate(dlg->nextUpdate->getDate());
@@ -210,25 +285,5 @@ pki_crl *db_crl::newItem(pki_x509 *cert)
 		crl = NULL;
 	}
 	delete dlg;
-	return crl;
-}
-
-void db_crl::showContextMenu(QContextMenuEvent *e, const QModelIndex &index)
-{
-	QMenu *menu = new QMenu(mainwin);
-	currentIdx = index;
-	QMenu *subExport;
-
-	menu->addAction(tr("Import"), this, SLOT(load()));
-	if (index != QModelIndex()) {
-		menu->addAction(tr("Rename"), this, SLOT(edit()));
-		subExport = menu->addMenu(tr("Export"));
-		subExport->addAction(tr("Clipboard"), this,
-					SLOT(pem2clipboard()));
-		subExport->addAction(tr("File"), this, SLOT(store()));
-		menu->addAction(tr("Delete"), this, SLOT(delete_ask()));
-	}
-	contextMenu(e, menu);
-	currentIdx = QModelIndex();
 	return;
 }
