@@ -11,30 +11,32 @@
 #include "db_x509.h"
 #include "pki_crl.h"
 #include "widgets/ExportDialog.h"
-#include <QtCore/QObject>
-#include <QtGui/QPixmap>
-#include <QtGui/QContextMenuEvent>
+#include <QObject>
+#include <QPixmap>
+#include <QContextMenuEvent>
 
 class db_crl: public db_x509name
 {
 		Q_OBJECT
 	protected:
 		QPixmap *crlicon;
+		dbheaderList getHeaders();
 	public:
 		db_crl(QString db, MainWindow *mw);
 		pki_base *newPKI(db_header_t *head = NULL);
 		void revokeCerts(pki_crl *crl);
 		void inToCont(pki_base *pki);
 		pki_base *insert(pki_base *item);
-		pki_crl *newItem(pki_x509 *cert);
-		void showContextMenu(QContextMenuEvent *e, const QModelIndex &index);
 		void removeSigner(pki_base *signer);
-	public slots:
-		void store();
+		void store(QModelIndex index);
 		void load();
 		void showPki(pki_base *pki);
-	signals:
 		void updateCertView();
+		void updateRevocations(pki_x509 *cert);
+
+	public slots:
+		void newItem();
+		void newItem(pki_x509 *cert);
 };
 
 #endif
