@@ -195,7 +195,7 @@ void db_key::showPki(pki_base *pki)
 		delete dlg;
 	}
 }
-exportType::etype db_key::clipboardFormat(QModelIndexList indexes)
+exportType::etype db_key::clipboardFormat(QModelIndexList indexes) const
 {
 	QList<exportType> types;
 	bool allPriv = true;
@@ -219,7 +219,9 @@ exportType::etype db_key::clipboardFormat(QModelIndexList indexes)
 			"pub", tr("SSH2 public"));
 	if (allPriv)
 		types << exportType(exportType::PEM_private, "pem",
-			tr("PEM private"));
+			tr("PEM private"))
+		      << exportType(exportType::PKCS8, "pk8",
+			"PKCS#8");
 
 	ExportDialog *dlg = new ExportDialog(mainwin,
 		tr("Export keys to Clipboard"), QString(), NULL,
@@ -297,14 +299,14 @@ void db_key::store(QModelIndex index)
 			break;
 		case exportType::PEM_private_encrypt:
 			enc = EVP_des_ede3_cbc();
-			/* fall */
+			/* fallthrough */
 		case exportType::PEM_private:
 			privkey->writeKey(fname, enc,
 				PwDialog::pwCallback, true);
 			break;
 		case exportType::PKCS8_encrypt:
 			enc = EVP_des_ede3_cbc();
-			/* fall */
+			/* fallthrough */
 		case exportType::PKCS8:
 			privkey->writePKCS8(fname, enc,
 				PwDialog::pwCallback, true);
