@@ -57,13 +57,13 @@ dbheaderList db_x509::getHeaders()
 	h <<	new dbheader(HD_cert_ca,	true, tr("CA"),
 			tr("reflects the basic Constraints extension")) <<
 		new dbheader(HD_cert_serial,	true, tr("Serial")) <<
-		new dbheader(HD_cert_md5fp,	false,tr("md5 fingerprint")) <<
-		new dbheader(HD_cert_sha1fp,	false,tr("sha1 fingerprint")) <<
-		new dbheader(HD_cert_sha256fp,	false,tr("sha256 fingerprint")) <<
+		new dbheader(HD_cert_md5fp,	false,tr("MD5 fingerprint")) <<
+		new dbheader(HD_cert_sha1fp,	false,tr("SHA1 fingerprint")) <<
+		new dbheader(HD_cert_sha256fp,	false,tr("SHA256 fingerprint")) <<
 		new dbheader(HD_cert_notBefore, false,tr("Start date"),
-				tr("not Before")) <<
+				tr("Not before")) <<
 		new dbheader(HD_cert_notAfter,	true, tr("Expiry date"),
-				tr("not After")) <<
+				tr("Not after")) <<
 		new dbheader(HD_cert_trust,	false,tr("Trust state")) <<
 		new dbheader(HD_cert_revocation,false, tr("Revocation")) <<
 		new dbheader(HD_cert_crl_expire,true, tr("CRL Expiration"));
@@ -307,6 +307,31 @@ QList<pki_x509*> db_x509::getCerts(bool onlyTrusted)
 	return c;
 }
 
+void db_x509::writeIndex(const QString fname, bool hierarchy)
+{
+	bool append = false;
+	if (hierarchy) {
+		FOR_ALL_pki(pki, pki_x509) {
+			if (pki->childCount()) {
+
+				QString newfname = fname + "." + pki->getIntName().replace(QRegExp("[^a-zA-Z0-9]"),QString(""));;
+
+				append = false;
+				foreach(pki_base *_child, pki->childItems) {
+					pki_x509 *child = static_cast<pki_x509*>(_child);
+					child->writeIndexEntry(newfname, append);
+					append = true;
+				}
+			}
+		}
+	} else {
+		FOR_ALL_pki(pki, pki_x509) {
+			pki->writeIndexEntry(fname, append);
+			append = true;
+		}
+	}
+}
+
 a1int db_x509::searchSerial(pki_x509 *signer)
 {
 	// returns the highest certificate serial
@@ -401,7 +426,7 @@ void db_x509::newItem()
 	emit connNewX509(dlg);
 	dlg->setCert();
 	pki_x509 *sigcert = get1SelectedCert();
-	dlg->defineSigner((pki_x509*)sigcert);
+	dlg->defineSigner((pki_x509*)sigcert, true);
 	if (dlg->exec()) {
 		newCert(dlg);
 	}
@@ -415,7 +440,7 @@ void db_x509::newCert(pki_x509req *req)
 	pki_x509 *sigcert = get1SelectedCert();
 	dlg->setCert();
 	dlg->defineRequest(req);
-	dlg->defineSigner(sigcert);
+	dlg->defineSigner(sigcert, true);
 	if (dlg->exec()) {
 		newCert(dlg);
 	}
@@ -439,7 +464,7 @@ void db_x509::newCert(pki_x509 *cert)
 	NewX509 *dlg = new NewX509(mainwin);
 	emit connNewX509(dlg);
 	dlg->setCert();
-	dlg->defineCert(cert);
+	dlg->fromX509super(cert, false);
 	if (dlg->exec()) {
 		newCert(dlg);
 	}
@@ -518,7 +543,7 @@ void db_x509::newCert(NewX509 *dlg)
 	dlg->initCtx(cert, signcert, NULL);
 	// if we can not sign
 	if (! signkey || signkey->isPubKey()) {
-		free(cert);
+		delete cert;
 		throw errorEx(tr("The key you selected for signing is not a private one."));
 	}
 
@@ -580,7 +605,6 @@ void db_x509::newCert(NewX509 *dlg)
 	}
 	if (tempkey != NULL)
 		delete(tempkey);
-		tempkey = NULL;
     }
 
     catch (errorEx &err) {
@@ -650,13 +674,13 @@ void db_x509::store(QModelIndexList list)
 			exportType(exportType::PEM_chain, "pem",
 				tr("PEM chain")) <<
 			exportType(exportType::PKCS7_chain, "p7b",
-				tr("PKCS #7 chain"));
+				tr("PKCS#7 chain"));
 	}
 
 	if (privkey && privkey->isPrivKey() && !privkey->isToken()) {
 		if (chain) {
 			usual << exportType(exportType::PKCS12_chain, "p12",
-				tr("PKCS #12 chain"));
+				tr("PKCS#12 chain"));
 			types << exportType(exportType::PKCS12, "p12",
 				"PKCS #12");
 		} else {
@@ -671,13 +695,15 @@ void db_x509::store(QModelIndexList list)
 	}
 	types << exportType() <<
 		exportType(exportType::PKCS7_trusted, "p7b",
-			tr("PKCS #7 trusted")) <<
+			tr("PKCS#7 trusted")) <<
 		exportType(exportType::PKCS7_all, "p7b",
-			tr("PKCS #7 all")) <<
+			tr("PKCS#7 all")) <<
 		exportType(exportType::PEM_trusted, "pem",
 			tr("PEM trusted")) <<
 		exportType(exportType::PEM_all, "pem",
-			tr("PEM all"));
+			tr("PEM all")) <<
+		exportType(exportType::Index, "txt",
+			tr("Certificate Index file"));
 
 	types = usual << exportType() << types;
 	ExportDialog *dlg = new ExportDialog(mainwin, tr("Certificate export"),
@@ -756,6 +782,14 @@ void db_x509::store(QModelIndexList list)
 			}
 			crt->writeCert(fname, true, true);
 			break;
+		case exportType::Index:
+			append = false;
+			foreach(QModelIndex idx, list) {
+				crt = static_cast<pki_x509*>(idx.internalPointer());
+				crt->writeIndexEntry(fname, append);
+				append = true;
+			}
+			break;
 		default:
 			exit(1);
 		}
@@ -1072,7 +1106,7 @@ void db_x509::toRequest(QModelIndex idx)
 		check_oom(req);
 		req->setIntName(cert->getIntName());
 		req->createReq(cert->getRefKey(), cert->getSubject(),
-			cert->getRefKey()->getDefaultMD(), cert->getV3ext());
+			cert->getDigest(), cert->getV3ext());
 		createSuccess(mainwin->reqs->insert(req));
 	}
 	catch (errorEx &err) {
