@@ -16,8 +16,12 @@
 #include <openssl/bio.h>
 #include <openssl/buffer.h>
 
-#if defined(Q_WS_MAC)
+#if defined(Q_OS_MAC)
+  #if QT_VERSION < 0x050000
 #include <QDesktopServices>
+  #else
+#include <QStandardPaths>
+  #endif
 #endif
 #include <QDir>
 #include <QFile>
@@ -31,12 +35,8 @@
 #include <QProgressBar>
 #include <QTextEdit>
 
-#ifdef WIN32
-#include <windows.h>
+#if defined(Q_OS_WIN32)
 #include <shlobj.h>
-#else
-/* for htons() */
-#include <netinet/in.h>
 #endif
 
 QPixmap *loadImg(const char *name )
@@ -49,7 +49,7 @@ QStringList getLibExtensions()
 	QStringList l;
 #if defined(_WIN32) || defined(USE_CYGWIN)
 	l << QString("*.dll") << QString("*.DLL");
-#elif defined(Q_WS_MAC)
+#elif defined(Q_OS_MAC)
 	l << QString("*.dylib") << QString("*.so");
 #else
 	l << QString("*.so");
@@ -64,7 +64,7 @@ QStringList getLibExtensions()
 
 QString getPrefix()
 {
-#ifdef WIN32
+#if defined(Q_OS_WIN32)
 	static char inst_dir[100] = "";
 	char *p;
 	ULONG dwLength = 100;
@@ -97,7 +97,7 @@ QString getPrefix()
 	lRc = RegCloseKey(hKey);
 	return QString(inst_dir);
 
-#elif defined(Q_WS_MAC)
+#elif defined(Q_OS_MAC)
 	// since this is platform-specific anyway,
 	// this is a more robust way to get the bundle directory
 	QDir bundleDir(qApp->applicationDirPath());
@@ -115,7 +115,7 @@ QString getPrefix()
 QString getHomeDir()
 {
 	QString hd;
-#ifdef WIN32
+#if defined(Q_OS_WIN32)
 	LPITEMIDLIST pidl = NULL;
 	TCHAR buf[255] = "";
 	if (SUCCEEDED(SHGetSpecialFolderLocation(NULL, CSIDL_PERSONAL, &pidl))) {
@@ -131,7 +131,7 @@ QString getHomeDir()
 QString getLibDir()
 {
 	QString hd;
-#ifdef WIN32
+#if defined(Q_OS_WIN32)
 	LPITEMIDLIST pidl = NULL;
 	TCHAR buf[255] = "";
 		if (SUCCEEDED(SHGetSpecialFolderLocation(NULL, CSIDL_SYSTEM, &pidl))) {
@@ -146,7 +146,7 @@ QString getLibDir()
 
 QString getDocDir()
 {
-#if defined(WIN32) || defined (Q_WS_MAC)
+#if defined(WIN32) || defined (Q_OS_MAC)
 	return getPrefix();
 #else
 	return QString(DOCDIR);
@@ -160,7 +160,7 @@ QString getDocDir()
 QString getUserSettingsDir()
 {
 	QString rv;
-#ifdef WIN32
+#if defined(Q_OS_WIN32)
 	LPITEMIDLIST pidl = NULL;
 	TCHAR buf[255] = "";
 	if (SUCCEEDED(SHGetSpecialFolderLocation(NULL, CSIDL_APPDATA, &pidl))) {
@@ -169,10 +169,17 @@ QString getUserSettingsDir()
 	rv = buf;
 	rv += QDir::separator();
 	rv += "xca";
-#elif defined(Q_WS_MAC)
+#elif defined(Q_OS_MAC)
+  #if QT_VERSION < 0x050000
 	rv = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
 	rv.insert(rv.count() - QCoreApplication::applicationName().count(),
 		QCoreApplication::organizationName());
+  #else
+	rv = QStandardPaths::writableLocation(
+			QStandardPaths::GenericDataLocation) + "/data/" +
+		QCoreApplication::organizationName() + "/" +
+		QCoreApplication::applicationName();
+  #endif
 #else
 	rv = QDir::homePath();
 	rv += QDir::separator();
@@ -197,7 +204,7 @@ QString getFullFilename(const QString & filename, const QString & selectedFilter
 
 QByteArray filename2bytearray(const QString &fname)
 {
-#ifdef WIN32
+#if defined(Q_OS_WIN32)
 	return fname.toLocal8Bit();
 #else
 	return fname.toUtf8();
@@ -206,7 +213,7 @@ QByteArray filename2bytearray(const QString &fname)
 
 QString filename2QString(const char *fname)
 {
-#ifdef WIN32
+#if defined(Q_OS_WIN32)
 	return QString::fromLocal8Bit(fname);
 #else
 	return QString::fromUtf8(fname);
@@ -244,7 +251,7 @@ QString asn1ToQString(const ASN1_STRING *str, bool quote)
 		case V_ASN1_BMPSTRING:
 			bmp = (unsigned short*)str->data;
 			for (i = 0; i < str->length/2; i++) {
-				unsigned short s = ntohs(bmp[i]);
+				unsigned short s = xntohs(bmp[i]);
 				qs += QString::fromUtf16(&s, 1);
 			}
 			break;
@@ -311,7 +318,7 @@ QString OBJ_obj2QString(const ASN1_OBJECT *a, int no_name)
 	char buf[512];
 	int len;
 
-	len = OBJ_obj2txt(buf, 256, a, no_name);
+	len = OBJ_obj2txt(buf, sizeof buf, a, no_name);
 	openssl_error();
 	return QString::fromLatin1(buf, len);
 }
@@ -393,49 +400,6 @@ void inc_progress_bar(int, int, void *p)
 	}
 }
 
-static long mem_ctrl(BIO *b, int cmd, long num, void *ptr)
-{
-	BUF_MEM *bm = (BUF_MEM *)b->ptr;
-	if (!bm->data || !(b->flags & BIO_FLAGS_MEM_RDONLY))
-		return BIO_s_mem()->ctrl(b, cmd, num, ptr);
-
-	switch (cmd) {
-	case BIO_C_FILE_SEEK:
-		if (num > (long)bm->max)
-			num = bm->max;
-		bm->data -= (bm->max - bm->length) - num;
-		bm->length = bm->max - num;
-	case BIO_C_FILE_TELL:
-		return bm->max - bm->length;
-	}
-	return BIO_s_mem()->ctrl(b, cmd, num, ptr);
-}
-
-void BIO_seekable_romem(BIO *b)
-{
-	static BIO_METHOD *mymeth = NULL;
-	static BIO_METHOD _meth;
-
-	if (!(b->flags & BIO_FLAGS_MEM_RDONLY) ||
-	     (b->method->type != BIO_TYPE_MEM))
-	{
-		return;
-	}
-	if (!mymeth) {
-		_meth = *BIO_s_mem();
-		_meth.ctrl = mem_ctrl;
-		mymeth = &_meth;
-	}
-	b->method = mymeth;
-}
-
-BIO *BIO_QBA_mem_buf(QByteArray &a)
-{
-	BIO *b = BIO_new_mem_buf(a.data(), a.size());
-	BIO_seekable_romem(b);
-	return b;
-}
-
 bool translate_dn = false;
 QMap<int, QString> dn_translations;
 
@@ -464,7 +428,7 @@ void dn_translations_setup()
 	dn_translations[NID_pkcs9_challengePassword] = QObject::tr("Challenge password");
 
 	dn_translations[NID_basic_constraints] = QObject::tr("Basic Constraints");
-	dn_translations[NID_subject_alt_name] = QObject::tr("subject alternative name");
+	dn_translations[NID_subject_alt_name] = QObject::tr("Subject alternative name");
 	dn_translations[NID_issuer_alt_name] = QObject::tr("issuer alternative name");
 	dn_translations[NID_subject_key_identifier] = QObject::tr("Subject key identifier");
 	dn_translations[NID_authority_key_identifier] = QObject::tr("Authority key identifier");
