@@ -1,32 +1,39 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2010 - 2012 Christian Hohnstaedt.
+ * Copyright (C) 2010 - 2015 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
 
+#ifndef __HEADERLIST_H
+#define __HEADERLIST_H
+
+#include <QString>
+#include <QList>
+#include <QAction>
+#include <QHeaderView>
+#include <openssl/objects.h>
 #include "db.h"
 #include "func.h"
-#include <QtCore/QString>
-#include <QtCore/QList>
-#include <QtGui/QAction>
-#include <QtGui/QHeaderView>
-#include <openssl/objects.h>
 
 #define HD_undef NID_undef
 #define HD_internal_name -2
 #define HD_subject_name -3
 #define HD_subject_hash -4
 #define HD_x509key_name -5
+#define HD_counter -6
+#define HD_x509_sigalg -7
 
 #define HD_cert_serial -10
 #define HD_cert_notBefore -11
 #define HD_cert_notAfter -12
 #define HD_cert_trust -13
-#define HD_cert_revokation -14
+#define HD_cert_revocation -14
 #define HD_cert_ca	 -15
 #define HD_cert_md5fp	 -16
 #define HD_cert_sha1fp   -17
+#define HD_cert_sha256fp -18
+#define HD_cert_crl_expire -19
 
 #define HD_req_signed -20
 #define HD_req_unstr_name -21
@@ -43,10 +50,11 @@
 #define HD_key_size -51
 #define HD_key_use  -52
 #define HD_key_passwd -53
+#define HD_key_curve -54
 
 class dbheader
 {
-    private:
+    protected:
 	void init()
 	{
 		id = HD_undef;
@@ -57,41 +65,44 @@ class dbheader
 		sortIndicator = -1;
 		type = hd_default;
 	}
+	QString name, tooltip;
+
     public:
 	enum hdr_type {
 		hd_default,
 		hd_x509name,
 		hd_v3ext,
+		hd_v3ext_ns,
 	};
 	int id;
 	bool show;
 	bool showDefault;
-	QString name;
-	QString tooltip;
+	virtual QString getName() { return name; }
+	virtual QString getTooltip() { return tooltip; }
 	QAction *action;
 	int size;
 	int visualIndex;
 	int sortIndicator;
 	enum hdr_type type;
 
+#if 1
 	dbheader(QString aname = QString())
 	{
 		init();
 		name = aname;
 	}
-	dbheader(int aid, bool ashow = false,
+#endif
+	dbheader(int aid, bool ashow,
 		QString aname = QString(), QString atip = QString())
 	{
 		init();
 		id = aid;
 		name = aname;
 		tooltip = atip;
-		if (id > 0 && name.isEmpty()) {
-			name = OBJ_nid2ln(aid);
-			tooltip = OBJ_nid2sn(aid);
-		}
 		show = showDefault = ashow;
 	}
+	virtual ~dbheader() { }
+
 	bool mustSave()
 	{
 		return  size != -1 ||
@@ -108,6 +119,7 @@ class dbheader
 	bool isNumeric()
 	{
 		switch (id) {
+		case HD_counter:
 		case HD_key_size:
 		case HD_key_use:
 		case HD_cert_serial:
@@ -116,6 +128,7 @@ class dbheader
 		case HD_subject_hash:
 		case HD_cert_md5fp:
 		case HD_cert_sha1fp:
+		case HD_cert_sha256fp:
 		case NID_subject_key_identifier:
 		case NID_authority_key_identifier:
 			return true;
@@ -158,10 +171,42 @@ class dbheader
 	}
 };
 
+class nid_dbheader : public dbheader
+{
+    private:
+	QString sn;
+
+    public:
+	nid_dbheader(int aid, enum hdr_type atype)
+		: dbheader(aid, aid == NID_commonName)
+	{
+		type = atype;
+		tooltip = dn_translations[id];
+		name = OBJ_nid2ln(id);
+		sn = OBJ_nid2sn(id);
+		if (tooltip.isEmpty())
+			tooltip = name;
+	}
+	QString getName()
+	{
+		return translate_dn ? tooltip : name;
+	}
+	QString getTooltip()
+	{
+		return QString("[%1] %2").arg(sn)
+			.arg(translate_dn ? name : tooltip);
+	}
+};
+
 class dbheaderList: public QList<dbheader*>
 {
     public:
 
+	dbheaderList(dbheader *h) :QList<dbheader*>() {
+		append(h);
+	}
+	dbheaderList() :QList<dbheader*>() {
+	}
 	QByteArray toData()
 	{
 		QByteArray ba;
@@ -202,3 +247,4 @@ class dbheaderList: public QList<dbheader*>
 		}
 	}
 };
+#endif
