@@ -28,6 +28,7 @@ pki_base *load_base::loadItem(QString s)
 		return NULL;
 	try {
 		pki->fload(s);
+		openssl_error();
 	}
 	catch (errorEx &err){
 		delete pki;
@@ -49,8 +50,8 @@ load_base::~load_base()
 load_key::load_key()
 	:load_base()
 {
-	filter = QObject::tr("PKI Keys ( *.pem *.der *.key );;"
-			"PKCS#8 Keys ( *.p8 *.pk8 );;"
+	filter = QObject::tr("PKI Keys ( *.pem *.der *.key );; "
+			"PKCS#8 Keys ( *.p8 *.pk8 );; "
 			"SSH Public Keys ( *.pub );;") + filter;
 	caption = QObject::tr("Import RSA key");
 }
@@ -64,8 +65,7 @@ pki_base * load_key::newItem()
 load_req::load_req()
 	:load_base()
 {
-	filter = QObject::tr("PKCS#10 CSR ( *.pem *.der *.csr );;"
-			"Netscape Request ( *.spkac *.spc );;") + filter;
+	filter = QObject::tr("PKCS#10 CSR ( *.pem *.der *.csr );; ") + filter;
 	caption = QObject::tr("Import Request");
 }
 
@@ -152,9 +152,9 @@ load_db::load_db()
 load_pkcs11::load_pkcs11()
 	:load_base()
 {
-#ifdef WIN32
+#if defined(Q_OS_WIN32)
 	filter = QObject::tr("PKCS#11 library ( *.dll );;") + filter;
-#elif defined(Q_WS_MAC)
+#elif defined(Q_OS_MAC)
 	filter = QObject::tr("PKCS#11 library ( *.dylib *.so );;") + filter;
 #else
 	filter = QObject::tr("PKCS#11 library ( *.so );;") + filter;
