@@ -7,6 +7,7 @@
 
 #include <signal.h>
 #include <QApplication>
+#include <QClipboard>
 #include <QTranslator>
 #include <QTextCodec>
 #include <QDir>
@@ -19,7 +20,8 @@
 #include "lib/db.h"
 #include "lib/main.h"
 #include "lib/entropy.h"
-#ifdef WIN32
+#if defined(Q_OS_WIN32)
+//For the segfault handler
 #include <windows.h>
 #endif
 
@@ -62,14 +64,14 @@ XCA_application::XCA_application(int &argc, char *argv[])
 			langAvail << QLocale(language);
 	}
 	setupLanguage(lang);
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
 	QStringList libp = libraryPaths();
 	libp.prepend(applicationDirPath() + "/../Plugins");
 	setLibraryPaths(libp);
 #endif
 
 	tableFont = QFont("Courier", QApplication::font().pointSize()
-#if defined(_WIN32) || defined(USE_CYGWIN)
+#if defined (Q_OS_WIN32)
 	+1
 #else
 	+2
@@ -116,9 +118,11 @@ void XCA_application::setupLanguage(QLocale l)
 		}
 	}
 	xcaTr->load(lang, "xca", getPrefix());
-
+	QLocale::setDefault(l);
 	installTranslator(qtTr);
 	installTranslator(xcaTr);
+	if (mainw)
+		mainw->initResolver();
 }
 
 void XCA_application::switchLanguage(QAction* a)
@@ -146,9 +150,9 @@ bool XCA_application::eventFilter(QObject *watched, QEvent *ev)
 	static int mctr;
 	QMouseEvent *me;
 	QStringList l;
+	XcaTreeView *treeview;
 	int key;
 
-	(void)watched;
 	switch (ev->type()) {
 	case QEvent::FileOpen:
 		l << static_cast<QFileOpenEvent *>(ev)->file();
@@ -169,6 +173,19 @@ bool XCA_application::eventFilter(QObject *watched, QEvent *ev)
 			entropy.add(key);
 		}
 		break;
+	case QEvent::MouseButtonPress:
+		me = static_cast<QMouseEvent *>(ev);
+		treeview = watched ?
+			dynamic_cast<XcaTreeView*>(watched->parent()) : NULL;
+
+		if ((watched == mainw || treeview) &&
+		    me->button() == Qt::MidButton &&
+		    QApplication::clipboard()->supportsSelection())
+		{
+			mainw->pastePem();
+			return true;
+		}
+		break;
 	default:
 		break;
 	}
@@ -261,7 +278,7 @@ int main_extract(int argc, char *argv[])
 
 char segv_data[1024];
 
-#ifdef WIN32
+#if defined(Q_OS_WIN32)
 static LONG CALLBACK w32_segfault(LPEXCEPTION_POINTERS e)
 {
 	if (e->ExceptionRecord->ExceptionCode == EXCEPTION_ACCESS_VIOLATION) {
@@ -286,8 +303,9 @@ int main( int argc, char *argv[] )
 {
 	int ret = 0, pkictr;
 	MainWindow *mw;
+	QDir d;
 
-#ifdef WIN32
+#if defined(Q_OS_WIN32)
 	SetUnhandledExceptionFilter(w32_segfault);
 #else
 	signal(SIGSEGV, segv_handler_gui);
@@ -296,6 +314,7 @@ int main( int argc, char *argv[] )
 	if (QString(argv[1]) == "extract") {
 		return main_extract(argc, argv);
 	}
+	d.mkpath(getUserSettingsDir());
 	XCA_application a(argc, argv);
 	mw = new MainWindow(NULL);
 	try {
