@@ -8,7 +8,7 @@
 #include "pkcs11.h"
 #include "pk11_attribute.h"
 #include "exception.h"
-#include <QtCore/QObject>
+#include <QObject>
 
 void pk11_attribute::load(slotid slot,
 			CK_SESSION_HANDLE sess, CK_OBJECT_HANDLE obj)
@@ -55,7 +55,7 @@ void pk11_attr_data::setValue(const unsigned char *ptr, unsigned long len)
 	((char*)attr.pValue)[len] = 0;
 }
 
-void pk11_attr_data::setBignum(BIGNUM *bn, bool consume)
+void pk11_attr_data::setConstBignum(const BIGNUM *bn)
 {
 	attr.ulValueLen = BN_num_bytes(bn);
 	if (attr.pValue)
@@ -63,6 +63,11 @@ void pk11_attr_data::setBignum(BIGNUM *bn, bool consume)
 	attr.pValue = malloc(attr.ulValueLen);
 	check_oom(attr.pValue);
 	attr.ulValueLen = BN_bn2bin(bn, (unsigned char *)attr.pValue);
+}
+
+void pk11_attr_data::setBignum(BIGNUM *bn, bool consume)
+{
+	setConstBignum(bn);
 	if (consume)
 		BN_free(bn);
 }
@@ -103,8 +108,10 @@ pk11_attlist::pk11_attlist(const pk11_attlist &a)
 
 pk11_attlist::~pk11_attlist()
 {
-	for (unsigned long i=0; i<attlen; i++)
+	for (unsigned long i=0; i<attlen; i++) {
+		memset(attributes[i].pValue, 0, attributes[i].ulValueLen);
 		free(attributes[i].pValue);
+	}
 	if (attributes)
 		free(attributes);
 }
