@@ -8,14 +8,14 @@
 #ifndef __PKI_PKCS11_ATTRIBUTE_H
 #define __PKI_PKCS11_ATTRIBUTE_H
 
-#include <QtCore/QString>
+#include <QString>
 #include <stdlib.h>
 #include <openssl/bn.h>
 #include "opensc-pkcs11.h"
 #include "exception.h"
 
 #define UTF8QSTRING(x,s) QString::fromUtf8((const char*)(x), s).trimmed()
-#define ASCIIQSTRING(x,s) QString::fromAscii((const char*)(x), s).trimmed()
+#define ASCIIQSTRING(x,s) QString::fromLatin1((const char*)(x), s).trimmed()
 
 class pk11_attlist;
 
@@ -148,6 +148,11 @@ public:
 	{
 		setBignum(bn, consume);
 	}
+	pk11_attr_data(unsigned long type, const BIGNUM *bn)
+		:pk11_attribute(type)
+	{
+		setConstBignum(bn);
+	}
 	pk11_attr_data(unsigned long type, unsigned long value)
 		:pk11_attribute(type)
 	{
@@ -167,8 +172,10 @@ public:
 	}
 	~pk11_attr_data()
 	{
-		if (attr.pValue)
+		if (attr.pValue) {
+			memset(attr.pValue, 0, attr.ulValueLen);
 			free(attr.pValue);
+		}
 	}
 	QString getText() const
 	{
@@ -187,6 +194,7 @@ public:
 				attr.ulValueLen, NULL);
 	}
 	void setBignum(BIGNUM *bn, bool consume=true);
+	void setConstBignum(const BIGNUM *bn);
 	void load(slotid slot,
 		CK_SESSION_HANDLE sess, CK_OBJECT_HANDLE obj);
 	void setValue(const unsigned char *ptr, unsigned long len);
