@@ -10,16 +10,28 @@
 
 #include "pkcs11_lib.h"
 #include "opensc-pkcs11.h"
-#include <QtCore/QStringList>
-#include <QtCore/QString>
-#include <QtCore/QList>
+#include <QStringList>
+#include <QString>
+#include <QList>
 
 #include <ltdl.h>
 
 #include "pk11_attribute.h"
 
-#define WAITCURSOR_START QApplication::setOverrideCursor(QCursor(Qt::WaitCursor))
-#define WAITCURSOR_END QApplication::restoreOverrideCursor()
+#define WAITCURSOR_START do { QApplication::setOverrideCursor(QCursor(Qt::WaitCursor)); ign_openssl_error(); } while(0);
+#define WAITCURSOR_END do { QApplication::restoreOverrideCursor(); ign_openssl_error(); } while(0);
+
+#define CALL_P11_C(l, func, ...) do { \
+	snprintf(segv_data, sizeof segv_data, "Crashed in %s in %s from %s:%d\n" \
+		"This looks like a bug in the PKC#11 library and not in XCA\n", \
+		#func, CCHAR((l)->filename()), __func__, __LINE__); \
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor)); \
+	rv = l->ptr()->func(__VA_ARGS__); \
+	segv_data[0] = 0; \
+	QApplication::restoreOverrideCursor(); \
+	ign_openssl_error(); \
+} while(0);
+
 
 class tkInfo
 {
@@ -152,6 +164,7 @@ class pkcs11
 				   CK_OBJECT_HANDLE object);
 		QList<CK_OBJECT_HANDLE> objectList(pk11_attlist &atts);
 		QString tokenLogin(QString name, bool so, bool force=false);
+		void getRandom();
 		void logout();
 		bool needsLogin(bool so);
 		void login(unsigned char *pin, unsigned long pinlen, bool so);
@@ -160,13 +173,14 @@ class pkcs11
 			unsigned char *pin, unsigned long pinLen);
 		CK_OBJECT_HANDLE createObject(pk11_attlist &attrs);
 		pk11_attr_data findUniqueID(unsigned long oclass);
-		pk11_attr_data generateRSAKey(QString name, unsigned long bits);
+		pk11_attr_data generateKey(QString name,
+			unsigned long ec_rsa_mech, unsigned long bits, int nid);
 		int deleteObjects(QList<CK_OBJECT_HANDLE> objects);
 		EVP_PKEY *getPrivateKey(EVP_PKEY *pub, CK_OBJECT_HANDLE obj);
 		int encrypt(int flen, const unsigned char *from,
-					unsigned char *to, int tolen);
+				unsigned char *to, int tolen, unsigned long m);
 		int decrypt(int flen, const unsigned char *from,
-					unsigned char *to, int tolen);
+				unsigned char *to, int tolen, unsigned long m);
 
 };
 
