@@ -11,6 +11,8 @@
 #include "exception.h"
 #include <QDir>
 
+#include "openssl_compat.h"
+
 QPixmap *pki_crl::icon = NULL;
 
 pki_crl::pki_crl(const QString name )
@@ -26,7 +28,7 @@ pki_crl::pki_crl(const QString name )
 
 void pki_crl::fromPEM_BIO(BIO *bio, QString name)
 {
-	X509_CRL*_crl;
+	X509_CRL *_crl;
 	_crl = PEM_read_bio_X509_CRL(bio, NULL, NULL, NULL);
 	openssl_error(name);
 	X509_CRL_free(crl);
@@ -69,8 +71,7 @@ void pki_crl::fload(const QString fname)
 				X509_CRL_free(_crl);
 			throw errorEx(tr("Unable to load the revocation list in file %1. Tried PEM and DER formatted CRL.").arg(fname));
 		}
-		if (crl)
-			X509_CRL_free(crl);
+		X509_CRL_free(crl);
 		crl = _crl;
 		setIntName(rmslashdot(fname));
 		pki_openssl_error();
@@ -80,8 +81,7 @@ void pki_crl::fload(const QString fname)
 
 QString pki_crl::getSigAlg()
 {
-	QString alg = OBJ_nid2ln(OBJ_obj2nid(crl->sig_alg->algorithm));
-	return alg;
+	return QString(OBJ_nid2ln(X509_CRL_get_signature_nid(crl)));
 }
 
 void pki_crl::createCrl(const QString d, pki_x509 *iss )
@@ -90,42 +90,46 @@ void pki_crl::createCrl(const QString d, pki_x509 *iss )
 	issuer = iss;
 	if (!iss)
 		my_error(tr("No issuer given"));
-	crl->crl->issuer = issuer->getSubject().get();
-	a1int version = 1; /* version 2 CRL */
-	crl->crl->version = version.get();
+	X509_CRL_set_version(crl, 1); /* version 2 CRL */
+	X509_CRL_set_issuer_name(crl, (X509_NAME*)issuer->getSubject().get0());
+
 	pki_openssl_error();
 }
 
 a1int pki_crl::getVersion()
 {
-	a1int a(crl->crl->version);
-	return a;
+	return a1int(X509_CRL_get_version(crl));
 }
 
 void pki_crl::setLastUpdate(const a1time &a)
 {
 	a1time t(a);
 	X509_CRL_set_lastUpdate(crl, t.get_utc());
+	pki_openssl_error();
 }
 
 void pki_crl::setNextUpdate(const a1time &a)
 {
 	a1time t(a);
 	X509_CRL_set_nextUpdate(crl, t.get_utc());
+	pki_openssl_error();
 }
 
 pki_crl::~pki_crl()
 {
 	X509_CRL_free(crl);
+	pki_openssl_error();
 }
 
 void pki_crl::d2i(QByteArray &ba)
 {
 	X509_CRL *c = (X509_CRL*)d2i_bytearray(D2I_VOID(d2i_X509_CRL), ba);
+	pki_openssl_error();
 	if (c) {
 		X509_CRL_free(crl);
 		crl = c;
 	}
+	pki_openssl_error();
 }
 
 QByteArray pki_crl::i2d()
@@ -168,15 +172,21 @@ void pki_crl::addV3ext(const x509v3ext &e)
 	pki_openssl_error();
 }
 
-bool pki_crl::visible()
+extList pki_crl::extensions() const
 {
 	extList el;
+	el.setStack(X509_CRL_get0_extensions(crl));
+	pki_openssl_error();
+	return el;
+}
+
+bool pki_crl::visible()
+{
 	if (pki_x509name::visible())
 		return true;
 	if (getSigAlg().contains(limitPattern))
 		return true;
-	el.setStack(crl->crl->extensions);
-	return el.search(limitPattern);
+	return extensions().search(limitPattern);
 }
 
 void pki_crl::sign(pki_key *key, const EVP_MD *md)
@@ -200,12 +210,10 @@ void pki_crl::writeCrl(const QString fname, bool pem)
 {
 	FILE *fp = fopen_write(fname);
 	if (fp != NULL) {
-		if (crl){
-			if (pem)
-				PEM_write_X509_CRL(fp, crl);
-			else
-				i2d_X509_CRL_fp(fp, crl);
-		}
+		if (pem)
+			PEM_write_X509_CRL(fp, crl);
+		else
+			i2d_X509_CRL_fp(fp, crl);
 		fclose(fp);
 		pki_openssl_error();
 	} else
@@ -223,34 +231,29 @@ BIO *pki_crl::pem(BIO *b, int format)
 
 a1time pki_crl::getLastUpdate()
 {
-	a1time a;
-	if (crl && crl->crl)
-		a.set(crl->crl->lastUpdate);
-	return a;
+	return a1time(X509_CRL_get0_lastUpdate(crl));
 }
 
 a1time pki_crl::getNextUpdate()
 {
-	a1time a;
-	if (crl && crl->crl)
-		a.set(crl->crl->nextUpdate);
-	return a;
+	return a1time(X509_CRL_get0_nextUpdate(crl));
 }
 
 int pki_crl::numRev()
 {
-	if (crl && crl->crl && crl->crl->revoked)
-		return sk_X509_REVOKED_num(crl->crl->revoked);
-	return 0;
+	STACK_OF(X509_REVOKED) *st = X509_CRL_get_REVOKED(crl);
+
+	return st ? sk_X509_REVOKED_num(st) : 0;
 }
 
 x509revList pki_crl::getRevList()
 {
 	x509revList ret;
 	int i, num = numRev();
+	STACK_OF(X509_REVOKED) *st = X509_CRL_get_REVOKED(crl);
 
 	for (i=0; i<num; i++) {
-		x509rev r(sk_X509_REVOKED_value(crl->crl->revoked, i));
+		x509rev r(sk_X509_REVOKED_value(st, i));
 		pki_openssl_error();
 		ret << r;
 	}
@@ -259,17 +262,13 @@ x509revList pki_crl::getRevList()
 
 x509name pki_crl::getSubject() const
 {
-	x509name x;
-	if (crl && crl->crl && crl->crl->issuer) {
-		x.set(crl->crl->issuer);
-	}
-	return x ;
+	return x509name(X509_CRL_get_issuer(crl));
 }
 
 bool pki_crl::verify(pki_key *key)
 {
 	bool ret=false;
-	if (crl && crl->crl && key) {
+	if (key) {
 		ret = (X509_CRL_verify(crl, key->getPubKey()) == 1);
 		pki_ign_openssl_error();
 	}
@@ -308,9 +307,8 @@ bool pki_crl::getCrlNumber(a1int *num)
 
 x509v3ext pki_crl::getExtByNid(int nid)
 {
-	extList el;
+	extList el = extensions();
 	x509v3ext e;
-	el.setStack(crl->crl->extensions);
 
 	for (int i=0; i< el.count(); i++){
 		if (el[i].nid() == nid) return el[i];
@@ -320,9 +318,7 @@ x509v3ext pki_crl::getExtByNid(int nid)
 
 QString pki_crl::printV3ext()
 {
-	extList el;
-	el.setStack(crl->crl->extensions);
-	QString text = el.getHtml("<br>");
+	QString text = extensions().getHtml("<br>");
 	pki_openssl_error();
 	return text;
 }
