@@ -14,6 +14,7 @@
 #include <openssl/err.h>
 #include <QMessageBox>
 
+#include "openssl_compat.h"
 
 pki_pkcs12::pki_pkcs12(const QString d, pki_x509 *acert, pki_evp *akey)
 	:pki_base(d)
@@ -61,10 +62,10 @@ pki_pkcs12::pki_pkcs12(const QString fname)
 		}
 		ign_openssl_error();
 		if (mycert) {
-			if (mycert->aux && mycert->aux->alias) {
-				alias = asn1ToQString(mycert->aux->alias);
-				alias = QString::fromUtf8(alias.toLatin1());
-			}
+			unsigned char *str = X509_alias_get0(mycert, NULL);
+			if (str)
+				alias = QString::fromUtf8((const char *)str);
+			alias = QString::fromUtf8(alias.toLatin1());
 			cert = new pki_x509(mycert);
 			if (alias.isEmpty()) {
 				cert->autoIntName();
