@@ -18,10 +18,12 @@
 #include <QDir>
 #include <openssl/rand.h>
 
+#include "openssl_compat.h"
+
 bool pki_x509::dont_colorize_expiries = false;
 bool pki_x509::disable_netscape = false;
 
-QPixmap *pki_x509::icon[6] = { NULL, NULL, NULL, NULL, NULL, NULL };
+QPixmap *pki_x509::icon[5];
 
 pki_x509::pki_x509(X509 *c)
 	:pki_x509super()
@@ -146,10 +148,7 @@ void pki_x509::init()
 
 void pki_x509::setSerial(const a1int &serial)
 {
-	if (cert->cert_info->serialNumber != NULL ) {
-		ASN1_INTEGER_free(cert->cert_info->serialNumber);
-	}
-	cert->cert_info->serialNumber = serial.get();
+	X509_set_serialNumber(cert, serial.get());
 	pki_openssl_error();
 }
 
@@ -176,7 +175,8 @@ a1int pki_x509::getIncCaSerial()
 	unsigned char buf[SERIAL_LEN];
 	if (!randomSerial)
 		return caSerial++;
-	RAND_pseudo_bytes(buf, SERIAL_LEN);
+
+	RAND_bytes(buf, SERIAL_LEN);
 	a1int serial;
 	serial.setRaw(buf, SERIAL_LEN);
 	return serial;
@@ -186,8 +186,8 @@ a1int pki_x509::hashInfo(const EVP_MD *md) const
 {
 	unsigned char digest[EVP_MAX_MD_SIZE];
 	unsigned len = 0;
-	if (!ASN1_item_digest(ASN1_ITEM_rptr(X509_CINF), md,
-				(char*)cert->cert_info,digest,&len))
+
+	if (!X509_digest(cert, md, digest, &len))
 		pki_openssl_error();
 	a1int a;
 	a.setRaw(digest,len);
@@ -399,31 +399,27 @@ a1time pki_x509::getNotAfter() const
 
 x509name pki_x509::getSubject() const
 {
-	x509name x(cert->cert_info->subject);
+	x509name x(X509_get_subject_name(cert));
 	pki_openssl_error();
 	return x;
 }
 
 x509name pki_x509::getIssuer() const
 {
-	x509name x(cert->cert_info->issuer);
+	x509name x(X509_get_issuer_name(cert));
 	pki_openssl_error();
 	return x;
 }
 
 void pki_x509::setSubject(const x509name &n)
 {
-	if (cert->cert_info->subject != NULL)
-		X509_NAME_free(cert->cert_info->subject);
-	cert->cert_info->subject = n.get();
+	X509_set_subject_name(cert, n.get());
 	pki_openssl_error();
 }
 
 void pki_x509::setIssuer(const x509name &n)
 {
-	if ((cert->cert_info->issuer) != NULL)
-		X509_NAME_free(cert->cert_info->issuer);
-	cert->cert_info->issuer = n.get();
+	X509_set_issuer_name(cert, n.get());
 	pki_openssl_error();
 }
 
@@ -497,34 +493,41 @@ void pki_x509::fromData(const unsigned char *p, db_header_t *head)
 		a1time revoked;
 		isRevoked = db::boolFromData(ba);
 		revoked.d2i(ba);
+		pki_openssl_error();
 		if (isRevoked) {
 			revocation.setDate(revoked);
 			revocation.setSerial(getSerial());
 		}
 	}
 	caSerial.setHex(db::stringFromData(ba));
+	pki_openssl_error();
 	caTemplate = db::stringFromData(ba);
 	crlDays = db::intFromData(ba);
 	crlExpiry.d2i(ba);
+	pki_openssl_error();
 	if (version > 1)
 		randomSerial = db::boolFromData(ba);
 	else
 		randomSerial = false;
 	if (version > 2)
 		crlNumber.setHex(db::stringFromData(ba));
+	pki_openssl_error();
 	if (version > 2 && version < 4) {
 		// load own revocation info, to tell daddy about it
 		a1time invalDate;
 		QString revoke_reason = db::stringFromData(ba);
 		invalDate.d2i(ba);
+		pki_openssl_error();
 		if (isRevoked) {
 			revocation.setReason(revoke_reason);
 			revocation.setInvalDate(invalDate);
 		}
 	}
+	pki_openssl_error();
 	if (version > 3) {
 		x509revList curr(revList);
 		revList.fromBA(ba);
+		pki_openssl_error();
 		revList.merge(curr);
 	}
 	if (ba.count() > 0) {
@@ -533,7 +536,6 @@ void pki_x509::fromData(const unsigned char *p, db_header_t *head)
 	pki_openssl_error();
 }
 
-
 QByteArray pki_x509::toData()
 {
 	QByteArray ba;
@@ -583,6 +585,42 @@ void pki_x509::writeCert(const QString fname, bool PEM, bool append)
 		fopen_error(fname);
 }
 
+void pki_x509::writeIndexEntry(FILE *fp)
+{
+	QString flag = NULL;
+	bool revoked = isRevoked();
+
+	if (revoked)
+		flag = "R";
+	else if (checkDate())
+		flag = "V";
+	else
+		flag = "E";
+
+	QString line = QString("%1\t%2\t%3\t%4\tunknown\t%5\n").arg(
+		flag, getNotAfter().toPlainUTC(), (revoked ? revocation.getDate().toPlainUTC() : ""),
+		getSerial().toHex(), QString(X509_NAME_oneline(getSubject().get(), NULL, 0)));
+
+	QByteArray ba = line.toUtf8();
+	fwrite(ba.constData(), ba.size(), 1, fp);
+}
+
+void pki_x509::writeIndexEntry(const QString fname, bool append)
+{
+	FILE *fp;
+	const char *p = "w";
+	if (append)
+		p = "a";
+	fp = fopen(QString2filename(fname), p);
+	if (fp != NULL) {
+		if (cert)
+			writeIndexEntry(fp);
+		fclose(fp);
+		pki_openssl_error();
+	} else
+		fopen_error(fname);
+}
+
 BIO *pki_x509::pem(BIO *b, int format)
 {
 	(void)format;
@@ -740,7 +778,7 @@ bool pki_x509::checkDate()
 extList pki_x509::getV3ext()
 {
 	extList el;
-	el.setStack(cert->cert_info->extensions);
+	el.setStack(X509_get0_extensions(cert));
 	return el;
 }
 
@@ -759,9 +797,9 @@ x509v3ext pki_x509::getExtByNid(int nid)
 	return el[i];
 }
 
-ASN1_OBJECT *pki_x509::sigAlg()
+int pki_x509::sigAlg()
 {
-	return cert->sig_alg->algorithm;
+	return X509_get_signature_nid(cert);
 }
 
 pki_x509 *pki_x509::getSigner()
@@ -917,7 +955,7 @@ QVariant pki_x509::getIcon(dbheader *hd)
 			return QVariant();
 		if (!ca)
 			return QVariant();
-		pixnum = 5;
+		pixnum = 4;
 		break;
 	case HD_internal_name:
 		k = getRefKey();
