@@ -39,10 +39,10 @@ class pki_x509 : public pki_x509super
 		x509rev revocation;
 
 	protected:
-		ASN1_OBJECT *sigAlg();
+		int sigAlg();
 
 	public:
-		static QPixmap *icon[6];
+		static QPixmap *icon[5];
 		static bool dont_colorize_expiries;
 		static bool disable_netscape;
 		x509revList revList;
@@ -74,6 +74,8 @@ class pki_x509 : public pki_x509super
 		void oldFromData(unsigned char *p, int size);
 		bool canSign();
 		void writeCert(const QString fname, bool PEM, bool append = false);
+		void writeIndexEntry(FILE *fp);
+		void writeIndexEntry(const QString fname, bool append = false);
 		bool verify(pki_x509 *signer);
 		bool verify_only(pki_x509 *signer);
 		pki_key *getPubKey() const;
