@@ -32,8 +32,8 @@ class pki_x509super : public pki_x509name
 		Q_OBJECT
 	protected:
 		pki_key *privkey;
-		virtual ASN1_OBJECT *sigAlg() {
-			return NULL;
+		virtual int sigAlg() {
+			return NID_undef;
 		}
 	public:
 		pki_x509super(const QString name = "");
@@ -52,10 +52,6 @@ class pki_x509super : public pki_x509name
 		};
 		virtual QString getSigAlg();
 		virtual const EVP_MD *getDigest();
-		virtual bool isSpki() const
-		{
-			return false;
-		}
 		pki_key *getRefKey() const;
 		void setRefKey(pki_key *ref);
 		void delRefKey(pki_key *ref);
