@@ -23,6 +23,7 @@ class x509name
 		x509name(STACK_OF(X509_NAME_ENTRY) *entries);
 		~x509name();
 		x509name &set(const X509_NAME *n);
+		x509name &set(const STACK_OF(X509_NAME_ENTRY) *entries);
 		QString oneLine(unsigned long flags = XN_FLAG_ONELINE) const;
 		int nid(int i) const;
 		QString getOid(int i) const;
@@ -40,6 +41,7 @@ class x509name
 		QString checkLength() const;
 		QString popEntryByNid(int nid);
 		X509_NAME *get() const;
+		const X509_NAME *get0() const;
 		QString getMostPopular() const;
 		QString taggedValues() const;
 		QString hash() const;
