@@ -2,12 +2,13 @@
 
 do_openssl()
 {
-openssl="openssl-1.0.2d"
-test -f "$openssl".tar.gz || curl http://www.openssl.org/source/"$openssl".tar.gz > "$openssl".tar.gz
+set -x
+read openssl < "`dirname $0`/../OpenSSL.version"
+test -f "$openssl".tar.gz || curl https://www.openssl.org/source/"$openssl".tar.gz > "$openssl".tar.gz
 tar zxf "$openssl".tar.gz
 (cd $openssl
- #./Configure darwin64-x86_64-cc shared --prefix=$INSTALL_DIR
- ./config shared --prefix=$INSTALL_DIR
+ ./Configure darwin64-x86_64-cc shared --prefix=$INSTALL_DIR
+ #./config shared --prefix=$INSTALL_DIR
  make && make install_sw
 )
 chmod 755 $INSTALL_DIR/lib/*.dylib
@@ -19,7 +20,7 @@ libtool="libtool-2.2.6b"
 test -f "$libtool".tar.gz || curl http://ftp.gnu.org/gnu/libtool/"$libtool".tar.gz > "$libtool".tar.gz
 tar zxf "$libtool".tar.gz
 cd "$libtool"
-export CFLAGS="-arch i386"
+#export CFLAGS="-arch i386"
 ./configure --prefix ${INSTALL_DIR}
 make && make install
 )}
@@ -32,10 +33,12 @@ XCA_DIR="`cd $XCA_DIR/.. && pwd`"
 export EXTRA_VERSION="$i"
 export INSTALL_DIR="`pwd`"/install
 export DYLD_LIBRARY_PATH=$INSTALL_DIR/lib
-export QTDIR=$HOME/src/install/Qt485
+export QTDIR=/Users/chris/Qt/5.9.1/clang_64
 
-do_libtool
-do_openssl
+if test -f build-libs; then
+  do_libtool
+  do_openssl
+fi
 
 XCA_BUILD="xca-macbuild"
 # configure XCA and build the DMG file
@@ -43,9 +46,11 @@ rm -rf "$XCA_BUILD"
 mkdir -p "$XCA_BUILD"
 cd "$XCA_BUILD"
 
-export CXXFLAGS="-arch i386 -I${INSTALL_DIR}/include -L${INSTALL_DIR}/lib -F$QTDIR"
+export CPPFLAGS="-I${INSTALL_DIR}/include -F$QTDIR"
+export CXXFLAGS="-F$QTDIR"
+export LDFLAGS="-L${INSTALL_DIR}/lib"
 
 (cd $XCA_DIR && ./bootstrap)
-$XCA_DIR/configure --with-openssl="$INSTALL_DIR" --with-qt="$QTDIR"
+$XCA_DIR/configure --with-openssl="$INSTALL_DIR" --with-qt=$QTDIR
 make -j5
 cp *.dmg ..
