@@ -1,16 +1,19 @@
 #!/bin/sh -e
 
+
 XCA_DIR="`dirname $0`"
 XCA_DIR="`cd $XCA_DIR/.. && pwd`"
 
+HOST=i686-w64-mingw32
+export CROSS="${HOST}-"
+TARGET=mingw
 LIBTOOL_DIR="libtool-2.2.6b"
 LIBTOOL_GZ="${LIBTOOL_DIR}".tar.gz
 LIBTOOL_DL="http://ftp.gnu.org/gnu/libtool/${LIBTOOL_GZ}"
 
-OPENSSL_DIR="openssl-1.0.2d"
+read OPENSSL_DIR < "`dirname $0`/../OpenSSL.version"
 OPENSSL_GZ="${OPENSSL_DIR}".tar.gz
-OPENSSL_DL="http://www.openssl.org/source/${OPENSSL_GZ}"
-OPENSSL_PATCH="$XCA_DIR/misc/openssl-1.0.0-mingw32-cross.patch"
+OPENSSL_DL="https://www.openssl.org/source/${OPENSSL_GZ}"
 
 unpack() {
   eval "dir=\${$1_DIR} gz=\${$1_GZ} dl=\${$1_DL} PATCH=\${$1_PATCH}"
@@ -28,13 +31,14 @@ unpack() {
 do_openssl()
 {(
 unpack OPENSSL
-sh ms/mingw32-cross.sh
+./Configure ${TARGET} shared --cross-compile-prefix="${CROSS}" --prefix="${INSTALL_DIR}"
+make && make install_sw
 )}
 
 do_libtool()
 {(
 unpack LIBTOOL
-./configure --host i586-mingw32msvc --prefix ${INSTALL_DIR}
+./configure --host ${HOST} --prefix ${INSTALL_DIR}
 make && make install
 )}
 
@@ -48,24 +52,11 @@ do_XCA()
   cp setup*.exe ..
 )}
 
-if ! test -f db_dump.exe; then
-  if ! curl "http://git.hohnstaedt.de/db_dump.exe" -o "db_dump.exe"; then
-    echo db_dump.exe missing
-    exit 1
-  fi
-fi
-
-if ! test -f mingwm10.dll; then
-  if ! zcat /usr/share/doc/mingw32-runtime/mingwm10.dll.gz >mingwm10.dll; then
-    echo missing mingwm10.dll
-    exit 1
-  fi
-fi
-
-
 XCA_BUILD="`pwd`"/xca_build
 export INSTALL_DIR=`pwd`/install
 
-do_openssl
-do_libtool
+if test -f build-libs; then
+  do_openssl
+  do_libtool
+fi
 do_XCA
