@@ -167,6 +167,11 @@ int MainWindow::init_database()
 
 	dbindex->setText(tr("Database") + ": " + dbfile);
 	load_engine();
+	hashBox hb(this);
+	if (hb.isInsecure()) {
+		XCA_WARN(tr("The currently used default hash '%1' is insecure. Please select at least 'SHA 224' for security reasons.").arg(hb.currentHashName()));
+		setOptions();
+	}
 	return ret;
 }
 
@@ -277,7 +282,9 @@ void MainWindow::default_database()
 	if (fp) {
 		QByteArray ba;
 		ba = filename2bytearray(fi.canonicalFilePath() + "\n");
-		fwrite(ba.constData(), ba.size(), 1, fp);
+		if (fwrite(ba.constData(), ba.size(), 1, fp)) {
+			/* IGNORE_RESULT */
+		}
 		fclose(fp);
 	}
 
