@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4:
  *
- * Copyright (C) 2001 - 2015 Christian Hohnstaedt.
+ * Copyright (C) 2001 - 2017 Christian Hohnstaedt.
  *
  * All rights reserved.
  */
@@ -19,24 +19,28 @@
 #include "lib/func.h"
 #include "lib/entropy.h"
 
-void MainWindow::cmd_version() {
-	fprintf(stderr, XCA_TITLE " Version " PACKAGE_VERSION "\n");
+void MainWindow::cmd_version()
+{
+	fprintf(stderr, XCA_TITLE " Version %s\n", version_str(false));
 	exitApp = 1;
 }
 
-void MainWindow::cmd_help(const char* msg) {
-	exitApp = 1;
-	fprintf(stderr, XCA_TITLE " Version " PACKAGE_VERSION "\n"
+void MainWindow::cmd_help(const char* msg)
+{
+	fprintf(stderr, XCA_TITLE " Version %s\n"
 		"\n"
 		" -v show version information and exit\n"
 		" -h shows this help screen and exit\n"
 		" -e <type>:<name>  extract entry.\n"
 		" -d expect the following argument to be the database name to use\n"
-		" -x Exit after processing all commandline options\n\n");
+		" -i expect the following argument to be the index file to generate\n"
+		" -I expect the following argument to be the base name the index file hierarchy to generate\n"
+		" -x Exit after processing all commandline options\n\n", version_str(false));
 
-	if(msg) {
+	if (msg) {
 		fprintf(stderr, "Cmdline Error: %s\n", msg);
 	}
+	exitApp = 1;
 }
 
 void MainWindow::about()
@@ -67,8 +71,8 @@ void MainWindow::about()
 	{
 		version = QString("<table border=0 width=500><tr>"
 				"<td>Compile time:</td>"
-				"<td>"OPENSSL_VERSION_TEXT"</td>"
-				"<td>QT version: "QT_VERSION_STR"</td>"
+				"<td>" OPENSSL_VERSION_TEXT "</td>"
+				"<td>QT version: " QT_VERSION_STR "</td>"
 				"</tr><tr>"
 				"<td>Run time:</td>"
 				"<td>%1</td>"
@@ -81,29 +85,25 @@ void MainWindow::about()
 	Entropy::seed_rng();
 	cont = QString(
 	"<p><h3><center><u>XCA</u></center></h3>"
-	"<p>Copyright 2001 - 2015 by Christian Hohnst&auml;dt\n"
-	"<p>Version: <b>" PACKAGE_VERSION "</b>"
-	"<p>%1<p>%2"
-	"<p>http://sourceforge.net/projects/xca"
+	"<p>Copyright 2001 - 2018 by Christian Hohnst&auml;dt\n"
+	"<p>Version: %4<p>%1<p>%2" /* commithash, Brainpool, OpenSSL & Qt Version */
+	"<p>http://hohnstaedt.de/xca"
 	"<p>Entropy strength: %3"
+	"<p>Installation path: <i>%5</i><br/>User settings path: <i>%6</i>"
 	"<hr><table border=0>"
 	"<tr><th align=left>Christian Hohnst&auml;dt</th><td><u>&lt;christian@hohnstaedt.de&gt;</u></td></tr>"
 	"<tr><td></td><td>Programming, Translation and Testing</td></tr>"
 	"<tr><th align=left>Kerstin Steinhauff</th><td><u>&lt;tine@kerstine.de&gt;</td></u></tr>"
 	"<tr><td></td><td>Arts and Graphics</td></tr>"
-	"</table><hr><center><u><b>General support</b></u></center>"
-	"<p><table>"
-	"<tr><td><b>Mark Foster</b></td><td><u>&lt;mark@foster.cc&gt;</u></td></tr>"
-	"<tr><td><b>Thorsten Weiss</b></td><td><u>&lt;weiss2@gmx.de&gt;</u></td></tr>"
-	"<tr><td><b>Oobj</b></td><td><u>&lt;www.oobj.com.br&gt;</u></td></tr>"
-	"<tr><td><b>Frank Isemann</b></td><td><u>&lt;isemannf@firflabs.net&gt;</u></td></tr>"
-	"<tr><td><b>Helmut Strobl</b></td><td><u>&lt;helmut.strobl@t-online.de&gt;</u></td></tr>"
 	"</table><hr><center><u><b>Maintained Translations</b></u></center>"
 	"<p><table>"
 	"<tr><td><b>German</b></td><td>Christian Hohnst&auml;dt &lt;christian@hohnstaedt.de&gt;</td></tr>"
 	"<tr><td><b>French</b></td><td>Patrick Monnerat &lt;Patrick.Monnerat@datasphere.ch&gt;</td></tr>"
 	"<tr><td><b>Croatian</b></td><td>Nevenko Bartolincic &lt;nevenko.bartolincic@gmail.com&gt;</td></tr>"
-	"</table>").arg(brainpool).arg(version).arg(Entropy::strength());
+	"<tr><td><b>Slovak</b></td><td>Slavko &lt;linux@slavino.sk&gt;</td></tr>"
+	"</table>").arg(brainpool).arg(version).arg(Entropy::strength())
+			.arg(version_str(true)).arg(getPrefix())
+			.arg(getUserSettingsDir());
 
 	about->setWindowTitle(XCA_TITLE);
 	ui.image->setPixmap( *keyImg );
@@ -121,11 +121,11 @@ void MainWindow::help()
 	ui.setupUi(h);
 
 	path = QString("file://");
-#ifdef WIN32
+#if defined(Q_OS_WIN32)
 	path += "/";
 #endif
 	path += getDocDir() + "/";
-#ifdef WIN32
+#if defined(Q_OS_WIN32)
 	path = path.replace("\\","/");
 #endif
 	uri = path + "xca.html";
