@@ -55,7 +55,6 @@ db_crl	*MainWindow::crls = NULL;
 
 NIDlist *MainWindow::eku_nid = NULL;
 NIDlist *MainWindow::dn_nid = NULL;
-NIDlist *MainWindow::aia_nid = NULL;
 
 QString MainWindow::mandatory_dn;
 QString MainWindow::explicit_dn;
@@ -80,6 +79,22 @@ void MainWindow::load_engine()
 	enableTokenMenu(pkcs11::loaded());
 }
 
+void MainWindow::initResolver()
+{
+	bool shown = false;
+	QString search;
+
+	if (resolver) {
+		shown = resolver->isVisible();
+		search = resolver->input->text();
+		delete resolver;
+	}
+	resolver = new OidResolver(NULL);
+	resolver->setWindowTitle(XCA_TITLE);
+	if (shown)
+		resolver->searchOid(search);
+}
+
 MainWindow::MainWindow(QWidget *parent)
 	:QMainWindow(parent)
 {
@@ -93,8 +108,7 @@ MainWindow::MainWindow(QWidget *parent)
 	setupUi(this);
 	setWindowTitle(XCA_TITLE);
 
-	resolver = new OidResolver(NULL);
-	resolver->setWindowTitle(XCA_TITLE);
+	initResolver();
 
 	wdList << keyButtons << reqButtons << certButtons <<
 		tempButtons <<	crlButtons;
@@ -126,11 +140,11 @@ MainWindow::MainWindow(QWidget *parent)
 
 	eku_nid = read_nidlist("eku.txt");
 	dn_nid = read_nidlist("dn.txt");
-	aia_nid = read_nidlist("aia.txt");
 
 	setAcceptDrops(true);
 
 	searchEdit = new QLineEdit();
+	searchEdit->setPlaceholderText(tr("Search"));
 
 	keyView->setMainwin(this, searchEdit);
 	reqView->setMainwin(this, searchEdit);
@@ -141,16 +155,21 @@ MainWindow::MainWindow(QWidget *parent)
 
 void MainWindow::dropEvent(QDropEvent *event)
 {
-	QList<QUrl> urls = event->mimeData()->urls();
-	QUrl u;
-	QStringList files;
+	if (event->mimeData()->hasUrls()) {
+		QList<QUrl> urls = event->mimeData()->urls();
+		QUrl u;
+		QStringList files;
 
-	foreach(u, urls) {
-		QString s = u.toLocalFile();
-		files << s;
+		foreach(u, urls) {
+			QString s = u.toLocalFile();
+			files << s;
+		}
+		openURLs(files);
+		event->acceptProposedAction();
+	} else if (event->mimeData()->hasText()) {
+		event->acceptProposedAction();
+		pastePem(event->mimeData()->text());
 	}
-	openURLs(files);
-	event->acceptProposedAction();
 }
 
 void MainWindow::openURLs(QStringList &files)
@@ -178,7 +197,10 @@ void MainWindow::openURLs()
 
 void MainWindow::dragEnterEvent(QDragEnterEvent *event)
 {
-	if (event->mimeData()->hasUrls())
+	if (event->mimeData()->hasFormat(X_XCA_DRAG_DATA))
+		return;
+
+	if (event->mimeData()->hasUrls() || event->mimeData()->hasText())
 		event->acceptProposedAction();
 }
 
@@ -202,10 +224,10 @@ NIDlist *MainWindow::read_nidlist(QString name)
 	NIDlist nl;
 	name = QDir::separator() + name;
 
-#ifndef WIN32
 	/* first try $HOME/xca/ */
 	nl = readNIDlist(getUserSettingsDir() + name);
-#if !defined(Q_WS_MAC)
+#if !defined(Q_OS_WIN32)
+#if !defined(Q_OS_MAC)
 	if (nl.count() == 0){
 		/* next is /etx/xca/... */
 		nl = readNIDlist(QString(ETC) + name);
@@ -242,21 +264,19 @@ void MainWindow::init_images()
 	pki_scard::icon[0] = loadImg("scard.png");
 	pki_x509req::icon[0] = loadImg("req.png");
 	pki_x509req::icon[1] = loadImg("reqkey.png");
-	pki_x509req::icon[2] = loadImg("spki.png");
-	pki_x509req::icon[3] = doneIco;
+	pki_x509req::icon[2] = doneIco;
 	pki_x509::icon[0] = loadImg("validcert.png");
 	pki_x509::icon[1] = loadImg("validcertkey.png");
 	pki_x509::icon[2] = loadImg("invalidcert.png");
 	pki_x509::icon[3] = loadImg("invalidcertkey.png");
-	pki_x509::icon[4] = loadImg("revoked.png");
-	pki_x509::icon[5] = doneIco;
+	pki_x509::icon[4] = doneIco;
 	pki_temp::icon = loadImg("template.png");
 	pki_crl::icon = loadImg("crl.png");
 }
 
 void MainWindow::read_cmdline(int argc, char *argv[])
 {
-	int cnt = 1, opt = 0, force_load = 0;
+	int cnt = 1, opt = 0, force_load = 0, export_index = 0;
 	char *arg = NULL;
 	exitApp = 0;
 	QStringList failed;
@@ -278,6 +298,12 @@ void MainWindow::read_cmdline(int argc, char *argv[])
 				case 'd':
 					force_load=1;
 					break;
+				case 'i':
+					export_index=1;
+					break;
+				case 'I':
+					export_index=2;
+					break;
 				case 'v':
 					cmd_version();
 					opt=0;
@@ -305,6 +331,10 @@ void MainWindow::read_cmdline(int argc, char *argv[])
 			if (changeDB(file) == 2)
 				exitApp = 1;
 			force_load = 0;
+		} else if (export_index) {
+			if (exportIndex(file, (export_index == 2)) == 2)
+				exitApp = 1;
+			export_index = 0;
 		} else {
 			int ret;
 			pki_multi *pki = probeAnything(file, &ret);
@@ -333,14 +363,15 @@ bool MainWindow::pastePem(QString text)
 {
 	bool success = false;
 	QByteArray pemdata = text.toLatin1();
-	BIO *b = BIO_QBA_mem_buf(pemdata);
-	check_oom(b);
+	if (pemdata.size() == 0)
+		return false;
+
 	pki_multi *pem = NULL;
 	ImportMulti *dlgi = NULL;
 	try {
 		pem = new pki_multi();
 		dlgi = new ImportMulti(this);
-		pem->fromPEM_BIO(b, QString("paste"));
+		pem->fromPEMbyteArray(pemdata, QString("paste"));
 		success = pem->count() != 0;
 		dlgi->addItem(pem);
 		pem = NULL;
@@ -353,7 +384,6 @@ bool MainWindow::pastePem(QString text)
 		delete dlgi;
 	if (pem)
 		delete pem;
-	BIO_free(b);
 	return success;
 }
 
@@ -380,12 +410,13 @@ void MainWindow::pastePem()
 	ui.button->setText(tr("Import PEM data"));
 	input->setWindowTitle(XCA_TITLE);
 	textbox->setPlainText(text);
-	if (input->exec())
+	if (input->exec()) {
 		text = textbox->toPlainText();
+		if (!text.isEmpty())
+			pastePem(text);
+	}
 	delete input;
 
-	if (!text.isEmpty())
-		pastePem(text);
 }
 
 void MainWindow::initToken()
@@ -413,7 +444,7 @@ void MainWindow::initToken()
 		if (ti.tokenInitialized()) {
 			ret = PwDialog::execute(&p, &pin, false);
 		} else {
-			p.setDescription(tr("Please enter the new SO PIN (PUK) of the token '%1'").
+			p.setDescription(tr("Please enter the new SO PIN (PUK) for the token '%1'").
 			arg(slotname) + "\n" + ti.pinInfo());
 			ret = PwDialog::execute(&p, &pin, true);
 		}
@@ -554,8 +585,6 @@ MainWindow::~MainWindow()
 		delete eku_nid;
 	if (dn_nid)
 		delete dn_nid;
-	if (aia_nid)
-		delete aia_nid;
 	delete dbindex;
 #ifdef MDEBUG
 	fprintf(stderr, "Memdebug:\n");
@@ -782,16 +811,15 @@ void MainWindow::Error(errorEx &err)
 {
 	if (err.isEmpty())
 		 return;
-	QString msg =  tr("The following error occured:") + "\n" + err.getString();
+	QString msg =  tr("The following error occurred:") + "\n" + err.getString();
 	xcaWarning box(NULL, msg);
 	box.addButton(QMessageBox::Apply)->setText(tr("Copy to Clipboard"));
 	box.addButton(QMessageBox::Ok);
 	if (box.exec() == QMessageBox::Apply) {
 		QClipboard *cb = QApplication::clipboard();
+		cb->setText(msg);
 		if (cb->supportsSelection())
 			cb->setText(msg, QClipboard::Selection);
-		else
-			cb->setText(msg);
 	}
 }
 
@@ -858,6 +886,50 @@ pki_multi *MainWindow::probeAnything(QString file, int *ret)
 	return pki;
 }
 
+void MainWindow::exportIndex()
+{
+	exportIndex(NULL, false);
+}
+
+void MainWindow::exportIndexHierarchy()
+{
+	exportIndex(NULL, true);
+}
+
+int MainWindow::exportIndex(QString fname, bool hierarchy)
+{
+	if (fname == NULL || fname.isEmpty()) {
+
+		QString filter = tr("Certificate Index ( index.txt )") + ";;" + tr("All files ( * )");
+
+		fname = QFileDialog::getSaveFileName(this, QString(), fname, filter, NULL);
+
+		if (fname.isEmpty())
+			return 1;
+
+		nativeSeparator(fname);
+	}
+
+	if (certs == NULL) {
+		open_default_db();
+		if (certs == NULL)
+			return 2;
+	}
+
+	bool needChangeView = hierarchy && !certs->treeview;
+	if (needChangeView) {
+		certView->changeView();
+	}
+
+	certs->writeIndex(fname, hierarchy);
+
+	if (needChangeView) {
+		certView->changeView();
+	}
+
+	return 0;
+}
+
 void MainWindow::generateDHparam()
 {
 	DH *dh = NULL;
@@ -866,6 +938,9 @@ void MainWindow::generateDHparam()
 	bool ok;
 	int num = QInputDialog::getDouble(this, XCA_TITLE, tr("Diffie-Hellman parameters are needed for different applications, but not handled by XCA.\nPlease enter the DH parameter bits"),
 		1024, 1024, 4096, 0, &ok);
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+	BN_GENCB *cb = NULL;
+#endif
 
 	if (!ok)
 		return;
@@ -883,7 +958,18 @@ void MainWindow::generateDHparam()
 		bar->setMinimum(0);
 		bar->setMaximum(100);
 		status->addPermanentWidget(bar, 1);
+
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+		cb = BN_GENCB_new();
+		if (!cb)
+			throw errorEx("");
+		BN_GENCB_set_old(cb, inc_progress_bar, bar);
+		dh = DH_new();
+		DH_generate_parameters_ex(dh, num, 2, cb);
+#else
 		dh = DH_generate_parameters(num, 2, inc_progress_bar, bar);
+#endif
+
 		status->removeWidget(bar);
 		openssl_error();
 
@@ -906,6 +992,10 @@ void MainWindow::generateDHparam()
 		DH_free(dh);
 	if (fp)
 		fclose(fp);
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+	if (cb)
+		BN_GENCB_free(cb);
+#endif
 	if (bar)
 		delete bar;
 }
@@ -947,6 +1037,12 @@ void MainWindow::keyPressEvent(QKeyEvent *e)
 			XCA_application::tableFont.setPointSize(siz -1);
 		}
 		break;
+	case Qt::Key_V:
+		if (e->modifiers() == Qt::ControlModifier) {
+			pastePem();
+			break;
+		}
+        /* FALLTHROUGH */
 	default:
 		QMainWindow::keyPressEvent(e);
 		return;
