@@ -86,6 +86,7 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		QStringList urlsToOpen;
 		int checkOldGetNewPass(Passwd &pass);
 		QString updateDbPassword(QString newdb, Passwd pass);
+		int exportIndex(QString fname, bool hierarchy);
 
 	protected:
 		void init_images();
@@ -109,7 +110,7 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		static QPixmap *keyImg, *csrImg, *certImg, *tempImg,
 				*nsImg, *revImg, *appIco, *scardImg,
 				*doneIco, *warnIco;
-		static NIDlist *eku_nid, *dn_nid, *aia_nid;
+		static NIDlist *eku_nid, *dn_nid;
 		static QString mandatory_dn;
 		static QString explicit_dn;
 		static QString explicit_dn_default;
@@ -145,6 +146,7 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		void setDefaultKey(QString def);
 		void load_history();
 		void update_history(QString file);
+		void initResolver();
 
 	public slots:
 		int init_database();
@@ -165,6 +167,8 @@ class MainWindow: public QMainWindow, public Ui::MainWindow
 		void openURLs(QStringList &files);
 		void openURLs();
 		void changeEvent(QEvent *event);
+		void exportIndex();
+		void exportIndexHierarchy();
 
 	protected slots:
 		void closeEvent(QCloseEvent * event);
