@@ -12,10 +12,10 @@
 #include "widgets/distname.h"
 #include "widgets/clicklabel.h"
 #include "lib/pkcs11.h"
-#include <QtGui/QLabel>
-#include <QtGui/QPushButton>
-#include <QtGui/QLineEdit>
-#include <QtCore/QStringList>
+#include <QLabel>
+#include <QPushButton>
+#include <QLineEdit>
+#include <QStringList>
 
 struct typelist {
 	const char *name;
@@ -43,19 +43,49 @@ class keyListItem
 	bool card;
 	QString printname;
 	slotid slot;
+	unsigned minKeySize;
+	unsigned maxKeySize;
+	unsigned long ec_flags;
+
 	keyListItem(pkcs11 *p11, slotid nslot, CK_MECHANISM_TYPE m)
 	{
-		// assert(m == CKM_RSA_PKCS_KEY_PAIR_GEN);
 		slot = nslot;
 		CK_MECHANISM_INFO mechinfo;
 		p11->mechanismInfo(slot, m, &mechinfo);
+
+		minKeySize = mechinfo.ulMinKeySize;
+		maxKeySize = mechinfo.ulMaxKeySize;
+		if (maxKeySize == 0) {
+			/* Fallback for libraries not filling in the maxKeySize */
+			maxKeySize = INT_MAX;
+		}
 		tkInfo ti = p11->tokenInfo(slot);
-		tl = typeList; //idx of EVP_PKEY_RSA
+		switch (m) {
+		case CKM_RSA_PKCS_KEY_PAIR_GEN:
+			tl = typeList; //idx of EVP_PKEY_RSA
+			break;
+		case CKM_DSA_KEY_PAIR_GEN:
+			tl = typeList +1;
+			break;
+#ifndef OPENSSL_NO_EC
+		case CKM_EC_KEY_PAIR_GEN:
+			tl = typeList +2;
+			CK_MECHANISM_INFO info;
+			p11->mechanismInfo(slot, m, &info);
+			ec_flags = info.flags & (CKF_EC_F_P | CKF_EC_F_2M);
+			if (!ec_flags) {
+				/* Fallback: Assume to support both for
+				 * libraries leaving this flag empty
+				 */
+				ec_flags = CKF_EC_F_P | CKF_EC_F_2M;
+			}
+#endif
+		}
 		printname = QString("%1 #%2 (%3 Key of %4 - %5 bits)").
 			arg(ti.label()).arg(ti.serial()).
 			arg(tl->name).
-			arg(mechinfo.ulMinKeySize).
-			arg(mechinfo.ulMaxKeySize);
+			arg(minKeySize).
+			arg(maxKeySize);
 		card = true;
 	}
 	keyListItem(const struct typelist *t=typeList)
@@ -64,6 +94,9 @@ class keyListItem
 		printname = QString(tl->name);
 		card = false;
 		slot = slotid();
+		minKeySize = 0;
+		maxKeySize = INT_MAX;
+		ec_flags = 0;
 	}
 	keyListItem(const keyListItem &k)
 	{
@@ -71,6 +104,9 @@ class keyListItem
 		printname = k.printname;
 		card = k.card;
 		slot = k.slot;
+		minKeySize = k.minKeySize;
+		maxKeySize = k.maxKeySize;
+		ec_flags = k.ec_flags;
 	}
 	int type()
 	{
@@ -89,7 +125,6 @@ NewKey::NewKey(QWidget *parent, QString name)
 {
 	static const char* const sizeList[] = { "1024", "2048", "4096" };
 	size_t i;
-	QStringList curve_x962, curve_other;
 	slotidList p11_slots;
 	QList<keyListItem> keytypes;
 
@@ -108,31 +143,7 @@ NewKey::NewKey(QWidget *parent, QString name)
 		keyListItem gk(typeList +i);
 		keytypes << gk;
 	}
-#ifndef OPENSSL_NO_EC
-	QString ec_default;
-	for (i = 0; i<pki_evp::num_curves; i++) {
-		const char *desc = pki_evp::curves[i].comment;
-		const char *sn = OBJ_nid2sn(pki_evp::curves[i].nid);
-
-		if (!sn)
-			continue;
-		if (desc == NULL)
-			desc = "---";
-		QString p = QString(sn) + ": " + desc;
-		if (pki_evp::curves[i].nid == defaultEcNid)
-			ec_default = p;
-		switch (pki_evp::curve_flags[i]) {
-			case CURVE_X962:  curve_x962  << p; break;
-			case CURVE_OTHER: curve_other << p; break;
-		}
-
-	}
-	curveBox->addItems(curve_x962);
-	curveBox->addItems(curve_other);
-	curveBox->setCurrentIndex(curveBox->findText(ec_default));
-	if (curveBox->currentIndex() == -1)
-		curveBox->setCurrentIndex(0);
-#endif
+	updateCurves();
 	keyLength->setEditText(QString::number(defaultSize) + " bit");
 	keyDesc->setFocus();
 	if (pkcs11::loaded()) try {
@@ -145,6 +156,16 @@ NewKey::NewKey(QWidget *parent, QString name)
 				keyListItem tk(&p11, slot, CKM_RSA_PKCS_KEY_PAIR_GEN);
 				keytypes << tk;
 			}
+			if (ml.contains(CKM_DSA_KEY_PAIR_GEN)) {
+				keyListItem tk(&p11, slot, CKM_DSA_KEY_PAIR_GEN);
+				keytypes << tk;
+			}
+#ifndef OPENSSL_NO_EC
+			if (ml.contains(CKM_EC_KEY_PAIR_GEN)) {
+				keyListItem tk(&p11, slot, CKM_EC_KEY_PAIR_GEN);
+				keytypes << tk;
+			}
+#endif
 		}
 	} catch (errorEx &err) {
 		p11_slots.clear();
@@ -159,15 +180,57 @@ NewKey::NewKey(QWidget *parent, QString name)
 	buttonBox->button(QDialogButtonBox::Ok)->setText(tr("Create"));
 }
 
+void NewKey::updateCurves(unsigned min, unsigned max, unsigned long ec_flags)
+{
+#ifndef OPENSSL_NO_EC
+	QString ec_default;
+	QStringList curve_x962, curve_other;
+	foreach(builtin_curve curve, pki_key::builtinCurves) {
+		const char *sn = OBJ_nid2sn(curve.nid);
+		QString comment = curve.comment;
+
+		if (!sn || curve.order_size < min || curve.order_size > max)
+			continue;
+		if (ec_flags) {
+			if ((curve.type & ec_flags) == 0)
+				continue;
+		}
+		if (comment.isEmpty())
+			comment = "---";
+		QString p = QString(sn) + ": " + comment;
+		if (curve.nid == defaultEcNid)
+			ec_default = p;
+		switch (curve.flags) {
+			case CURVE_X962:  curve_x962  << p; break;
+			case CURVE_OTHER: curve_other << p; break;
+		}
+	}
+	curveBox->clear();
+	curveBox->addItems(curve_x962);
+	curveBox->addItems(curve_other);
+	curveBox->setCurrentIndex(curveBox->findText(ec_default));
+	if (curveBox->currentIndex() == -1)
+		curveBox->setCurrentIndex(0);
+#else
+	(void)min; (void)max; (void)ec_flags;
+#endif
+}
+
 void NewKey::on_keyType_currentIndexChanged(int idx)
 {
 	bool curve_enabled;
+	keyListItem ki = keyType->itemData(idx).value<keyListItem>();
 
-	curve_enabled = (typeList[idx].type == EVP_PKEY_EC);
+	curve_enabled = (ki.type() == EVP_PKEY_EC);
 	curveBox->setVisible(curve_enabled);
 	curveLabel->setVisible(curve_enabled);
 	keySizeLabel->setVisible(!curve_enabled);
 	keyLength->setVisible(!curve_enabled);
+
+	rememberDefault->setEnabled(!ki.card);
+	if (curve_enabled && ki.card) {
+		updateCurves(ki.minKeySize, ki.maxKeySize, ki.ec_flags);
+	}
 }
 
 static keyListItem currentKey(QComboBox *keyType)
@@ -193,7 +256,7 @@ int NewKey::getKeysize()
 int NewKey::getKeyCurve_nid()
 {
 	if (getKeytype() != EVP_PKEY_EC)
-		return -1;
+		return NID_undef;
 	QString desc = curveBox->currentText();
 	desc.replace(QRegExp("^(X9.62) "), "");
 	desc.replace(QRegExp(":.*"), "");
@@ -242,7 +305,7 @@ int NewKey::setDefault(QString def)
 	if (type == -1)
 		return -2;
 	if (type == EVP_PKEY_EC) {
-		nid = OBJ_txt2nid(sl[1].toAscii());
+		nid = OBJ_txt2nid(sl[1].toLatin1());
 		if (nid == NID_undef)
 			return -3;
 		defaultEcNid = nid;
