@@ -32,7 +32,7 @@ NewX509::NewX509(QWidget *parent)
 	int i;
 	eku_nid = *MainWindow::eku_nid;
 	dn_nid = *MainWindow::dn_nid;
-	aia_nid = *MainWindow::aia_nid;
+	aia_nid << OBJ_sn2nid("OCSP") << OBJ_sn2nid("caIssuers");
 	attr_nid << NID_pkcs9_unstructuredName << NID_pkcs9_challengePassword;
 	foreach(QString dn, MainWindow::explicit_dn.split(","))
 		expl_dn_nid << OBJ_sn2nid(CCHAR(dn));
@@ -390,8 +390,8 @@ void NewX509::defineRequest(pki_x509req *req)
 	on_fromReqCB_clicked();
 }
 
-/* Preset all values from another request to create a aimilar one */
-void NewX509::fromX509super(pki_x509super *cert_or_req)
+/* Preset all values from another request to create a similar one */
+void NewX509::fromX509super(pki_x509super *cert_or_req, bool applyTemp)
 {
 	pki_temp *temp = new pki_temp("");
 	temp->fromCert(cert_or_req);
@@ -414,7 +414,7 @@ void NewX509::fromX509super(pki_x509super *cert_or_req)
 		if (signer == cert) {
 			foreignSignRB->setChecked(false);
 		} else if (signer) {
-			defineSigner(signer);
+			defineSigner(signer, applyTemp);
 		}
 		notBefore->setDate(cert->getNotBefore());
 		notAfter->setDate(cert->getNotAfter());
@@ -431,23 +431,8 @@ void NewX509::fromX509super(pki_x509super *cert_or_req)
 
 }
 
-/* Preset all values from another cert to create a aimilar one */
-void NewX509::defineCert(pki_x509 *cert)
-{
-	fromX509super(cert);
-
-	pki_x509 *signer = cert->getSigner();
-	if (signer == cert) {
-		foreignSignRB->setChecked(false);
-	} else if (signer) {
-		defineSigner(signer);
-	}
-	notBefore->setDate(cert->getNotBefore());
-	notAfter->setDate(cert->getNotAfter());
-}
-
 /* Preset the signing certificate */
-void NewX509::defineSigner(pki_x509 *defcert)
+void NewX509::defineSigner(pki_x509 *defcert, bool applyTemp)
 {
 	int index;
 	// suggested from: Andrey Brindeew <abr@abr.pp.ru>
@@ -458,13 +443,12 @@ void NewX509::defineSigner(pki_x509 *defcert)
 		if ((index = certList->findText(name)) >= 0) {
 			certList->setCurrentIndex(index);
 		}
-		if (!defcert->getTemplate().isEmpty()) {
+		if (applyTemp && !defcert->getTemplate().isEmpty()) {
 			on_applyTemplate_clicked();
 		}
 	}
 }
 
-
 static int lb2int(QListWidget *lb)
 {
 	int i, x=0, c=lb->count();
@@ -1110,10 +1094,13 @@ void NewX509::on_tabWidget_currentChanged(int tab)
 
 QString NewX509::mandatoryDnRemain()
 {
-	QStringList dnl = MainWindow::mandatory_dn.split(",");
+	QStringList remain, dnl = MainWindow::mandatory_dn.split(",");
 	x509name n;
 	int i;
 
+	if (MainWindow::mandatory_dn.isEmpty())
+		return QString();
+
 	if (fromReqCB->isChecked() && !reqSubChange->isChecked())
 		n = getSelectedReq()->getSubject();
 	else
@@ -1124,7 +1111,12 @@ QString NewX509::mandatoryDnRemain()
 		if (j>=0)
 			dnl.removeAt(j);
 	}
-	return dnl.join(",");
+	if (dnl.size() == 0)
+		return QString();
+
+	foreach(QString x, dnl)
+		remain << QString(OBJ_sn2ln(x.toLatin1()));
+	return QString("'%1'").arg(remain.join("','"));
 }
 
 void NewX509::gotoTab(int tab)
