@@ -18,6 +18,7 @@
 #include "lib/x509v3ext.h"
 #include "lib/func.h"
 
+#include "lib/openssl_compat.h"
 
 x509v3ext NewX509::getBasicConstraints()
 {
@@ -54,12 +55,7 @@ x509v3ext NewX509::getAuthKeyIdent()
 	if (!authKey->isChecked() || !authKey->isEnabled())
 		return ext;
 
-	QString x = "keyid,issuer:always";
-	if (ext_ctx.issuer_cert && X509_get_ext_by_NID(ext_ctx.issuer_cert,
-				NID_subject_key_identifier, -1) != -1)
-	{
-		x = "keyid:always,issuer:always";
-	}
+	QString x = "keyid,issuer";
 	ext.create(NID_authority_key_identifier, x, &ext_ctx);
 	return ext;
 }
@@ -198,6 +194,7 @@ extList NewX509::getAdvanced()
 	extList elist;
 	long err_line=0;
 	STACK_OF(X509_EXTENSION) **sk, *sk_tmp = NULL;
+	const STACK_OF(X509_EXTENSION) *csk;
 	const char *ext_name = "default";
 	int ret, start;
 
@@ -223,8 +220,8 @@ extList NewX509::getAdvanced()
 	}
 
 	if (ext_ctx.subject_cert) {
-		sk = &ext_ctx.subject_cert->cert_info->extensions;
-		start = *sk ? sk_X509_EXTENSION_num(*sk) : 0;
+		csk = X509_get0_extensions(ext_ctx.subject_cert);
+		start = csk? sk_X509_EXTENSION_num(csk): 0;
 	} else {
 		sk = &sk_tmp;
 		start = 0;
@@ -232,12 +229,21 @@ extList NewX509::getAdvanced()
 
 	X509V3_set_nconf(&ext_ctx, conf);
 
-	if (X509V3_EXT_add_nconf_sk(conf, &ext_ctx, (char *)ext_name, sk)) {
-		openssl_error();
+	if (ext_ctx.subject_cert) {
+		if (X509V3_EXT_add_nconf(conf, &ext_ctx,
+				(char *) ext_name, ext_ctx.subject_cert))
+			openssl_error();
+		csk = X509_get0_extensions(ext_ctx.subject_cert);
+	} else {
+		if (X509V3_EXT_add_nconf_sk(conf, &ext_ctx, (char *)ext_name, sk))
+			openssl_error();
+		csk = *sk;
 	}
-	elist.setStack(*sk, start);
-	if (sk == &sk_tmp)
+
+	elist.setStack(csk, start);
+	if (!ext_ctx.subject_cert)
 		sk_X509_EXTENSION_pop_free(sk_tmp, X509_EXTENSION_free);
+
 	X509V3_set_nconf(&ext_ctx, NULL);
 	NCONF_free(conf);
 	BIO_free(bio);
@@ -317,12 +323,12 @@ extList NewX509::getExtDuplicates()
 {
 	int i, start, cnt, n1, n;
 	x509v3ext e;
-	STACK_OF(X509_EXTENSION) *sk;
+	const STACK_OF(X509_EXTENSION) *sk;
 	extList el_dup, el;
 	QString olist;
 
 	if (ext_ctx.subject_cert) {
-		sk = ext_ctx.subject_cert->cert_info->extensions;
+		sk = X509_get0_extensions(ext_ctx.subject_cert);
 	} else
 		return el_dup;
 
