@@ -8,26 +8,30 @@
 #include "hashBox.h"
 #include "lib/base.h"
 
+/* SHA-256 as default */
+#define DEFAULT_MD_IDX 4
+
+/* SHA 1 and below are insecure */
+#define INSECURE_MD 2
 
 static struct {
 	const char *name;
 	const EVP_MD *md;
 } hashalgos[] = {
 	{ "MD 5", EVP_md5() },
+	{ "RIPEMD 160", EVP_ripemd160() },
 	{ "SHA 1", EVP_sha1() },
 	{ "SHA 224", EVP_sha224() },
 	{ "SHA 256", EVP_sha256() },
 	{ "SHA 384", EVP_sha384() },
 	{ "SHA 512", EVP_sha512() },
-	{ "RIPEMD 160", EVP_ripemd160() },
 };
 
 QString hashBox::default_md = QString();
 
 void hashBox::resetDefault()
 {
-	/* SHA1 */
-	default_md = QString(hashalgos[1].name);
+	default_md = QString(hashalgos[DEFAULT_MD_IDX].name);
 }
 
 hashBox::hashBox(QWidget *parent)
@@ -42,7 +46,22 @@ void hashBox::setKeyType(int type)
 	key_type = type;
 }
 
-const EVP_MD *hashBox::currentHash()
+int hashBox::currentHashIdx() const
+{
+	QString hash = currentText();
+	for (unsigned i=0; i<ARRAY_SIZE(hashalgos); i++) {
+		if (hash == hashalgos[i].name)
+			return i;
+	}
+	return DEFAULT_MD_IDX;
+}
+
+bool hashBox::isInsecure() const
+{
+	return currentHashIdx() <= INSECURE_MD;
+}
+
+const EVP_MD *hashBox::currentHash() const
 {
 #if OPENSSL_VERSION_NUMBER < 0x10000000L
 	switch(key_type) {
@@ -54,12 +73,10 @@ const EVP_MD *hashBox::currentHash()
 #endif
 	}
 #endif
-	QString hash = currentText();
-	for (unsigned i=0; i<ARRAY_SIZE(hashalgos); i++) {
-		if (hash == hashalgos[i].name)
-			return hashalgos[i].md;
-	}
-	return hashalgos[4].md; /* SHA-256 as fallback */
+	unsigned i= hashBox::currentHashIdx();
+	if (i >= ARRAY_SIZE(hashalgos))
+		i = DEFAULT_MD_IDX;
+	return hashalgos[i].md;
 }
 
 void hashBox::setCurrentString(QString md)
@@ -152,11 +169,18 @@ void hashBox::setCurrentMD(const EVP_MD *md)
 void hashBox::setupHashes(QList<int> nids)
 {
 	QString md = currentText();
+	int mdtype;
+
 	if (!wanted_md.isEmpty())
 		md = wanted_md;
 	clear();
 	for (unsigned i=0; i<ARRAY_SIZE(hashalgos); i++) {
-		if (nids.contains(hashalgos[i].md->type)) {
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+		mdtype = EVP_MD_type(hashalgos[i].md);
+#else
+		mdtype = hashalgos[i].md->type;
+#endif
+		if (nids.contains(mdtype)) {
 			addItem(QString(hashalgos[i].name));
 		}
 	}
@@ -176,7 +200,7 @@ void hashBox::setupAllHashes()
 	setCurrentString(md);
 }
 
-QString hashBox::currentHashName()
+QString hashBox::currentHashName() const
 {
 	return currentText();
 }
